import pytest
from aioresponses.core import aioresponses

from keba_keenergy_api.api import KebaKeEnergyAPI
from keba_keenergy_api.constants import HeatPumpCompressorUseNightSpeed
from keba_keenergy_api.constants import HeatPumpHasCompressorFailure
from keba_keenergy_api.constants import HeatPumpHasPassiveCooling
from keba_keenergy_api.constants import HeatPumpHasSourceActuatorFailure
from keba_keenergy_api.constants import HeatPumpHasSourceFailure
from keba_keenergy_api.constants import HeatPumpHasSourcePressureFailure
from keba_keenergy_api.constants import HeatPumpHasThreePhaseFailure
from keba_keenergy_api.constants import HeatPumpHasVFDFailure
from keba_keenergy_api.constants import HeatPumpHeatRequest
from keba_keenergy_api.constants import HeatPumpOperatingMode
from keba_keenergy_api.error import APIError


@pytest.mark.happy
class TestHappyPathHeatPumpSection:
    @pytest.mark.asyncio
    async def test_get_name(self) -> None:
        with aioresponses() as mock_keenergy_api:
            mock_keenergy_api.post(
                "http://mocked-host/var/readWriteVars",
                payload=[
                    {
                        "name": "APPL.CtrlAppl.sParam.heatpump[0].param.name",
                        "attributes": {
                            "longText": "Name",
                        },
                        "value": "WPS26",
                    },
                ],
                headers={"Content-Type": "application/json;charset=utf-8"},
            )

            client: KebaKeEnergyAPI = KebaKeEnergyAPI(host="mocked-host")
            data: str = await client.heat_pump.get_name()

            assert isinstance(data, str)
            assert data == "WPS26"

            mock_keenergy_api.assert_called_once_with(
                url="http://mocked-host/var/readWriteVars",
                data='[{"name": "APPL.CtrlAppl.sParam.heatpump[0].param.name", "attr": "1"}]',
                method="POST",
                auth=None,
                ssl=False,
            )

    @pytest.mark.asyncio
    @pytest.mark.parametrize(
        ("human_readable", "payload_value", "expected_value"),
        [(True, 1, "flow"), (False, 1, 1)],
    )
    async def test_get_state(
        self,
        human_readable: bool,  # noqa: FBT001
        payload_value: int,
        expected_value: int | str,
    ) -> None:
        with aioresponses() as mock_keenergy_api:
            mock_keenergy_api.post(
                "http://mocked-host/var/readWriteVars",
                payload=[
                    {
                        "name": "APPL.CtrlAppl.sParam.heatpump[0].values.heatpumpState",
                        "attributes": {
                            "formatId": "fmtHPState",
                            "longText": "State",
                            "lowerLimit": "0",
                            "unitId": "Enum",
                            "upperLimit": "32767",
                        },
                        "value": f"{payload_value}",
                    },
                ],
                headers={"Content-Type": "application/json;charset=utf-8"},
            )

            client: KebaKeEnergyAPI = KebaKeEnergyAPI(host="mocked-host")
            data: int | str = await client.heat_pump.get_state(human_readable=human_readable)

            assert isinstance(data, (int | str))
            assert data == expected_value

            mock_keenergy_api.assert_called_once_with(
                url="http://mocked-host/var/readWriteVars",
                data='[{"name": "APPL.CtrlAppl.sParam.heatpump[0].values.heatpumpState", "attr": "1"}]',
                method="POST",
                auth=None,
                ssl=False,
            )

    @pytest.mark.asyncio
    @pytest.mark.parametrize(
        ("human_readable", "payload_value", "expected_value"),
        [
            (True, 9, "flushing"),
            (True, 5, "pressure_equalization"),
            (True, 21, "pressure_equalization"),
        ],
    )
    async def test_get_substate(
        self,
        human_readable: bool,  # noqa: FBT001
        payload_value: int,
        expected_value: int | str,
    ) -> None:
        with aioresponses() as mock_keenergy_api:
            mock_keenergy_api.post(
                "http://mocked-host/var/readWriteVars",
                payload=[
                    {
                        "name": "APPL.CtrlAppl.sParam.heatpump[0].values.heatpumpSubState",
                        "attributes": {
                            "formatId": "fmtHPSubState",
                            "longText": "Substate",
                            "unitId": "Enum",
                            "upperLimit": "32767",
                            "lowerLimit": "0",
                        },
                        "value": f"{payload_value}",
                    },
                ],
                headers={"Content-Type": "application/json;charset=utf-8"},
            )

            client: KebaKeEnergyAPI = KebaKeEnergyAPI(host="mocked-host")
            data: int | str = await client.heat_pump.get_substate(human_readable=human_readable)

            assert isinstance(data, (int | str))
            assert data == expected_value

            mock_keenergy_api.assert_called_once_with(
                url="http://mocked-host/var/readWriteVars",
                data='[{"name": "APPL.CtrlAppl.sParam.heatpump[0].values.heatpumpSubState", "attr": "1"}]',
                method="POST",
                auth=None,
                ssl=False,
            )

    @pytest.mark.asyncio
    @pytest.mark.parametrize(
        ("human_readable", "payload_value", "expected_value"),
        [(True, 0, "off"), (False, 2, 2)],
    )
    async def test_get_operating_mode(
        self,
        human_readable: bool,  # noqa: FBT001
        payload_value: int,
        expected_value: str,
    ) -> None:
        with aioresponses() as mock_keenergy_api:
            mock_keenergy_api.post(
                "http://mocked-host/var/readWriteVars",
                payload=[
                    {
                        "name": "APPL.CtrlAppl.sParam.heatpump[0].param.operatingMode",
                        "attributes": {
                            "formatId": "fmtHPOpMode",
                            "longText": "Operating mode",
                            "lowerLimit": "0",
                            "unitId": "Enum",
                            "upperLimit": "2",
                        },
                        "value": f"{payload_value}",
                    },
                ],
                headers={"Content-Type": "application/json;charset=utf-8"},
            )

            client: KebaKeEnergyAPI = KebaKeEnergyAPI(host="mocked-host")
            data: int | str = await client.heat_pump.get_operating_mode(human_readable=human_readable)

            assert isinstance(data, (int | str))
            assert data == expected_value

            mock_keenergy_api.assert_called_once_with(
                url="http://mocked-host/var/readWriteVars",
                data='[{"name": "APPL.CtrlAppl.sParam.heatpump[0].param.operatingMode", "attr": "1"}]',
                method="POST",
                auth=None,
                ssl=False,
            )

    @pytest.mark.asyncio
    @pytest.mark.parametrize(
        ("operating_mode", "expected_value"),
        [("off", 0), ("ON", 1), (HeatPumpOperatingMode.BACKUP.value, 2)],
    )
    async def test_set_operating_mode(
        self,
        operating_mode: int | str,
        expected_value: int,
    ) -> None:
        with aioresponses() as mock_keenergy_api:
            mock_keenergy_api.post(
                "http://mocked-host/var/readWriteVars?action=set",
                payload={},
                headers={"Content-Type": "application/json;charset=utf-8"},
            )

            client: KebaKeEnergyAPI = KebaKeEnergyAPI(host="mocked-host")
            await client.heat_pump.set_operating_mode(operating_mode)

            mock_keenergy_api.assert_called_once_with(
                url="http://mocked-host/var/readWriteVars?action=set",
                data='[{"name": "APPL.CtrlAppl.sParam.heatpump[0].param.operatingMode", "value": "%s"}]'  # noqa: UP031
                % expected_value,
                method="POST",
                auth=None,
                ssl=False,
            )

    @pytest.mark.asyncio
    @pytest.mark.parametrize(
        ("human_readable", "payload_value", "expected_value"),
        [
            (True, "true", "on"),
            (True, "false", "off"),
            (False, "true", 1),
            (False, "false", 0),
        ],
    )
    async def test_get_compressor_use_night_speed(
        self,
        human_readable: bool,  # noqa: FBT001
        payload_value: int,
        expected_value: str,
    ) -> None:
        with aioresponses() as mock_keenergy_api:
            mock_keenergy_api.post(
                "http://mocked-host/var/readWriteVars",
                payload=[
                    {
                        "name": "APPL.CtrlAppl.sParam.heatpump[0].HeatPumpPowerCtrl.param.useDayNightSpeed",
                        "attributes": {
                            "longText": "Day/Night switch",
                        },
                        "value": f"{payload_value}",
                    },
                ],
                headers={"Content-Type": "application/json;charset=utf-8"},
            )

            client: KebaKeEnergyAPI = KebaKeEnergyAPI(host="mocked-host")
            data: int | str = await client.heat_pump.get_compressor_use_night_speed(human_readable=human_readable)

            assert isinstance(data, (int | str))
            assert data == expected_value

            mock_keenergy_api.assert_called_once_with(
                url="http://mocked-host/var/readWriteVars",
                data='[{"name": "APPL.CtrlAppl.sParam.heatpump[0].HeatPumpPowerCtrl.param.useDayNightSpeed", "attr": "1"}]',  # noqa: E501
                method="POST",
                auth=None,
                ssl=False,
            )

    @pytest.mark.asyncio
    @pytest.mark.parametrize(
        ("mode", "expected_value"),
        [
            ("off", "0"),
            ("ON", "1"),
            (HeatPumpCompressorUseNightSpeed.ON.value, "1"),
            (HeatPumpCompressorUseNightSpeed.OFF.value, "0"),
        ],
    )
    async def test_set_compressor_use_night_speed(
        self,
        mode: int | str,
        expected_value: str,
    ) -> None:
        with aioresponses() as mock_keenergy_api:
            mock_keenergy_api.post(
                "http://mocked-host/var/readWriteVars?action=set",
                payload={},
                headers={"Content-Type": "application/json;charset=utf-8"},
            )

            client: KebaKeEnergyAPI = KebaKeEnergyAPI(host="mocked-host")
            await client.heat_pump.set_compressor_use_night_speed(mode)

            mock_keenergy_api.assert_called_once_with(
                url="http://mocked-host/var/readWriteVars?action=set",
                data='[{"name": "APPL.CtrlAppl.sParam.heatpump[0].HeatPumpPowerCtrl.param.useDayNightSpeed", "value": "%s"}]'  # noqa: E501, UP031
                % expected_value,
                method="POST",
                auth=None,
                ssl=False,
            )

    @pytest.mark.asyncio
    async def test_get_compressor_night_speed(self) -> None:
        with aioresponses() as mock_keenergy_api:
            mock_keenergy_api.post(
                "http://mocked-host/var/readWriteVars",
                payload=[
                    {
                        "name": "APPL.CtrlAppl.sParam.heatpump[0].HeatPumpPowerCtrl.param.maxPowerScaledNight",
                        "attributes": {
                            "formatId": "fmt3p0",
                            "longText": "Max. pwr. limit night",
                            "unitId": "Pct100",
                            "upperLimit": "1",
                            "lowerLimit": "0.5",
                        },
                        "value": "0.5",
                    },
                ],
                headers={"Content-Type": "application/json;charset=utf-8"},
            )

            client: KebaKeEnergyAPI = KebaKeEnergyAPI(host="mocked-host")
            data: float = await client.heat_pump.get_compressor_night_speed()

            assert isinstance(data, float)
            assert data == 0.5  # noqa: PLR2004

            mock_keenergy_api.assert_called_once_with(
                url="http://mocked-host/var/readWriteVars",
                data='[{"name": "APPL.CtrlAppl.sParam.heatpump[0].HeatPumpPowerCtrl.param.maxPowerScaledNight", "attr": "1"}]',  # noqa: E501
                method="POST",
                auth=None,
                ssl=False,
            )

    @pytest.mark.asyncio
    async def test_set_compressor_night_speed(self) -> None:
        with aioresponses() as mock_keenergy_api:
            mock_keenergy_api.post(
                "http://mocked-host/var/readWriteVars?action=set",
                payload={},
                headers={"Content-Type": "application/json;charset=utf-8"},
            )

            client: KebaKeEnergyAPI = KebaKeEnergyAPI(host="mocked-host")
            await client.heat_pump.set_compressor_night_speed(0.6)

            mock_keenergy_api.assert_called_once_with(
                url="http://mocked-host/var/readWriteVars?action=set",
                data='[{"name": "APPL.CtrlAppl.sParam.heatpump[0].HeatPumpPowerCtrl.param.maxPowerScaledNight", "value": "0.6"}]',  # noqa: E501
                method="POST",
                auth=None,
                ssl=False,
            )

    @pytest.mark.asyncio
    async def test_get_min_compressor_night_speed(self) -> None:
        with aioresponses() as mock_keenergy_api:
            mock_keenergy_api.post(
                "http://mocked-host/var/readWriteVars",
                payload=[
                    {
                        "name": "APPL.CtrlAppl.sParam.heatpump[0].HeatPumpPowerCtrl.param.maxPowerScaledNight",
                        "attributes": {
                            "formatId": "fmt3p0",
                            "longText": "Max. pwr. limit night",
                            "unitId": "Pct100",
                            "upperLimit": "1",
                            "lowerLimit": "0.5",
                        },
                        "value": "0.5",
                    },
                ],
                headers={"Content-Type": "application/json;charset=utf-8"},
            )

            client: KebaKeEnergyAPI = KebaKeEnergyAPI(host="mocked-host")
            data: float = await client.heat_pump.get_min_compressor_night_speed()

            assert isinstance(data, float)
            assert data == 0.5  # noqa: PLR2004

            mock_keenergy_api.assert_called_once_with(
                url="http://mocked-host/var/readWriteVars",
                data='[{"name": "APPL.CtrlAppl.sParam.heatpump[0].HeatPumpPowerCtrl.param.maxPowerScaledNight", "attr": "1"}]',  # noqa: E501
                method="POST",
                auth=None,
                ssl=False,
            )

    @pytest.mark.asyncio
    async def test_get_max_compressor_night_speed(self) -> None:
        with aioresponses() as mock_keenergy_api:
            mock_keenergy_api.post(
                "http://mocked-host/var/readWriteVars",
                payload=[
                    {
                        "name": "APPL.CtrlAppl.sParam.heatpump[0].HeatPumpPowerCtrl.param.maxPowerScaledNight",
                        "attributes": {
                            "formatId": "fmt3p0",
                            "longText": "Max. pwr. limit night",
                            "unitId": "Pct100",
                            "upperLimit": "1",
                            "lowerLimit": "0.5",
                        },
                        "value": "0.5",
                    },
                ],
                headers={"Content-Type": "application/json;charset=utf-8"},
            )

            client: KebaKeEnergyAPI = KebaKeEnergyAPI(host="mocked-host")
            data: float = await client.heat_pump.get_max_compressor_night_speed()

            assert isinstance(data, float)
            assert data == 1

            mock_keenergy_api.assert_called_once_with(
                url="http://mocked-host/var/readWriteVars",
                data='[{"name": "APPL.CtrlAppl.sParam.heatpump[0].HeatPumpPowerCtrl.param.maxPowerScaledNight", "attr": "1"}]',  # noqa: E501
                method="POST",
                auth=None,
                ssl=False,
            )

    @pytest.mark.asyncio
    async def test_get_circulation_pump_speed(self) -> None:
        with aioresponses() as mock_keenergy_api:
            mock_keenergy_api.post(
                "http://mocked-host/var/readWriteVars",
                payload=[
                    {
                        "name": "APPL.CtrlAppl.sParam.heatpump[0].CircPump.values.setValueScaled",
                        "attributes": {
                            "formatId": "fmt3p0",
                            "longText": "Circulation pump",
                            "lowerLimit": "0.0",
                            "unitId": "Pct100",
                            "upperLimit": "1",
                        },
                        "value": "0.5",
                    },
                ],
                headers={"Content-Type": "application/json;charset=utf-8"},
            )

            client: KebaKeEnergyAPI = KebaKeEnergyAPI(host="mocked-host")
            data: float = await client.heat_pump.get_circulation_pump_speed()

            assert isinstance(data, float)
            assert data == 0.5  # noqa: PLR2004

            mock_keenergy_api.assert_called_once_with(
                url="http://mocked-host/var/readWriteVars",
                data='[{"name": "APPL.CtrlAppl.sParam.heatpump[0].CircPump.values.setValueScaled", "attr": "1"}]',
                method="POST",
                auth=None,
                ssl=False,
            )

    @pytest.mark.asyncio
    async def test_get_source_pump_speed(self) -> None:
        with aioresponses() as mock_keenergy_api:
            mock_keenergy_api.post(
                "http://mocked-host/var/readWriteVars",
                payload=[
                    {
                        "name": "APPL.CtrlAppl.sParam.heatpump[0].Source.values.setValueScaled",
                        "attributes": {
                            "formatId": "fmt3p0",
                            "longText": "Source",
                            "unitId": "Pct100",
                            "upperLimit": "1",
                            "lowerLimit": "0.0",
                        },
                        "value": "0.44875106",
                    },
                ],
                headers={"Content-Type": "application/json;charset=utf-8"},
            )

            client: KebaKeEnergyAPI = KebaKeEnergyAPI(host="mocked-host")
            data: float = await client.heat_pump.get_source_pump_speed()

            assert isinstance(data, float)
            assert data == 0.45  # noqa: PLR2004

            mock_keenergy_api.assert_called_once_with(
                url="http://mocked-host/var/readWriteVars",
                data='[{"name": "APPL.CtrlAppl.sParam.heatpump[0].Source.values.setValueScaled", "attr": "1"}]',
                method="POST",
                auth=None,
                ssl=False,
            )

    @pytest.mark.asyncio
    async def test_get_flow_temperature(self) -> None:
        with aioresponses() as mock_keenergy_api:
            mock_keenergy_api.post(
                "http://mocked-host/var/readWriteVars",
                payload=[
                    {
                        "name": "APPL.CtrlAppl.sParam.heatpump[0].TempHeatFlow.values.actValue",
                        "attributes": {
                            "formatId": "fmtTemp",
                            "longText": "Inflow temp.",
                            "unitId": "Temp",
                        },
                        "value": "24.200001",
                    },
                ],
                headers={"Content-Type": "application/json;charset=utf-8"},
            )

            client: KebaKeEnergyAPI = KebaKeEnergyAPI(host="mocked-host")
            data: float = await client.heat_pump.get_flow_temperature()

            assert isinstance(data, float)
            assert data == 24.2  # noqa: PLR2004

            mock_keenergy_api.assert_called_once_with(
                url="http://mocked-host/var/readWriteVars",
                data='[{"name": "APPL.CtrlAppl.sParam.heatpump[0].TempHeatFlow.values.actValue", "attr": "1"}]',
                method="POST",
                auth=None,
                ssl=False,
            )

    @pytest.mark.asyncio
    async def test_get_return_flow_temperature(self) -> None:
        with aioresponses() as mock_keenergy_api:
            mock_keenergy_api.post(
                "http://mocked-host/var/readWriteVars",
                payload=[
                    {
                        "name": "APPL.CtrlAppl.sParam.heatpump[0].TempHeatReflux.values.actValue",
                        "attributes": {
                            "formatId": "fmtTemp",
                            "longText": "Reflux temp.",
                            "unitId": "Temp",
                        },
                        "value": "23.200001",
                    },
                ],
                headers={"Content-Type": "application/json;charset=utf-8"},
            )

            client: KebaKeEnergyAPI = KebaKeEnergyAPI(host="mocked-host")
            data: float = await client.heat_pump.get_return_flow_temperature()

            assert isinstance(data, float)
            assert data == 23.2  # noqa: PLR2004

            mock_keenergy_api.assert_called_once_with(
                url="http://mocked-host/var/readWriteVars",
                data='[{"name": "APPL.CtrlAppl.sParam.heatpump[0].TempHeatReflux.values.actValue", "attr": "1"}]',
                method="POST",
                auth=None,
                ssl=False,
            )

    @pytest.mark.asyncio
    async def test_get_source_input_temperature(self) -> None:
        with aioresponses() as mock_keenergy_api:
            mock_keenergy_api.post(
                "http://mocked-host/var/readWriteVars",
                payload=[
                    {
                        "name": "APPL.CtrlAppl.sParam.heatpump[0].TempSourceIn.values.actValue",
                        "attributes": {
                            "formatId": "fmtTemp",
                            "longText": "Source in temp.",
                            "unitId": "Temp",
                        },
                        "value": "22.700001",
                    },
                ],
                headers={"Content-Type": "application/json;charset=utf-8"},
            )

            client: KebaKeEnergyAPI = KebaKeEnergyAPI(host="mocked-host")
            data: float = await client.heat_pump.get_source_input_temperature()

            assert isinstance(data, float)
            assert data == 22.7  # noqa: PLR2004

            mock_keenergy_api.assert_called_once_with(
                url="http://mocked-host/var/readWriteVars",
                data='[{"name": "APPL.CtrlAppl.sParam.heatpump[0].TempSourceIn.values.actValue", "attr": "1"}]',
                method="POST",
                auth=None,
                ssl=False,
            )

    @pytest.mark.asyncio
    async def test_get_source_output_temperature(self) -> None:
        with aioresponses() as mock_keenergy_api:
            mock_keenergy_api.post(
                "http://mocked-host/var/readWriteVars",
                payload=[
                    {
                        "name": "APPL.CtrlAppl.sParam.heatpump[0].TempSourceOut.values.actValue",
                        "attributes": {
                            "formatId": "fmtTemp",
                            "longText": "Source out temp.",
                            "unitId": "Temp",
                        },
                        "value": "24.6",
                    },
                ],
                headers={"Content-Type": "application/json;charset=utf-8"},
            )

            client: KebaKeEnergyAPI = KebaKeEnergyAPI(host="mocked-host")
            data: float = await client.heat_pump.get_source_output_temperature()

            assert isinstance(data, float)
            assert data == 24.6  # noqa: PLR2004

            mock_keenergy_api.assert_called_once_with(
                url="http://mocked-host/var/readWriteVars",
                data='[{"name": "APPL.CtrlAppl.sParam.heatpump[0].TempSourceOut.values.actValue", "attr": "1"}]',
                method="POST",
                auth=None,
                ssl=False,
            )

    @pytest.mark.asyncio
    async def test_get_compressor_input_temperature(self) -> None:
        with aioresponses() as mock_keenergy_api:
            mock_keenergy_api.post(
                "http://mocked-host/var/readWriteVars",
                payload=[
                    {
                        "name": "APPL.CtrlAppl.sParam.heatpump[0].TempCompressorIn.values.actValue",
                        "attributes": {
                            "formatId": "fmtTemp",
                            "longText": "Comp. in temp.",
                            "unitId": "Temp",
                        },
                        "value": "26.4",
                    },
                ],
                headers={"Content-Type": "application/json;charset=utf-8"},
            )

            client: KebaKeEnergyAPI = KebaKeEnergyAPI(host="mocked-host")
            data: float = await client.heat_pump.get_compressor_input_temperature()

            assert isinstance(data, float)
            assert data == 26.4  # noqa: PLR2004

            mock_keenergy_api.assert_called_once_with(
                url="http://mocked-host/var/readWriteVars",
                data='[{"name": "APPL.CtrlAppl.sParam.heatpump[0].TempCompressorIn.values.actValue", "attr": "1"}]',
                method="POST",
                auth=None,
                ssl=False,
            )

    @pytest.mark.asyncio
    async def test_get_compressor_output_temperature(self) -> None:
        with aioresponses() as mock_keenergy_api:
            mock_keenergy_api.post(
                "http://mocked-host/var/readWriteVars",
                payload=[
                    {
                        "name": "APPL.CtrlAppl.sParam.heatpump[0].TempCompressorOut.values.actValue",
                        "attributes": {
                            "formatId": "fmtTemp",
                            "longText": "Comp. out temp.",
                            "unitId": "Temp",
                        },
                        "value": "26.5",
                    },
                ],
                headers={"Content-Type": "application/json;charset=utf-8"},
            )

            client: KebaKeEnergyAPI = KebaKeEnergyAPI(host="mocked-host")
            data: float = await client.heat_pump.get_compressor_output_temperature()

            assert isinstance(data, float)
            assert data == 26.5  # noqa: PLR2004

            mock_keenergy_api.assert_called_once_with(
                url="http://mocked-host/var/readWriteVars",
                data='[{"name": "APPL.CtrlAppl.sParam.heatpump[0].TempCompressorOut.values.actValue", "attr": "1"}]',
                method="POST",
                auth=None,
                ssl=False,
            )

    @pytest.mark.asyncio
    async def test_get_compressor_speed(self) -> None:
        with aioresponses() as mock_keenergy_api:
            mock_keenergy_api.post(
                "http://mocked-host/var/readWriteVars",
                payload=[
                    {
                        "name": "APPL.CtrlAppl.sParam.heatpump[0].Compressor.values.setValueScaled",
                        "attributes": {
                            "formatId": "fmtTemp",
                            "longText": "Compressor",
                            "lowerLimit": "0.0",
                            "unitId": "Pct100",
                            "upperLimit": "1",
                        },
                        "value": "0.3",
                    },
                ],
                headers={"Content-Type": "application/json;charset=utf-8"},
            )

            client: KebaKeEnergyAPI = KebaKeEnergyAPI(host="mocked-host")
            data: float = await client.heat_pump.get_compressor_speed()

            assert isinstance(data, float)
            assert data == 0.3  # noqa: PLR2004

            mock_keenergy_api.assert_called_once_with(
                url="http://mocked-host/var/readWriteVars",
                data='[{"name": "APPL.CtrlAppl.sParam.heatpump[0].Compressor.values.setValueScaled", "attr": "1"}]',
                method="POST",
                auth=None,
                ssl=False,
            )

    @pytest.mark.asyncio
    async def test_get_condenser_temperature(self) -> None:
        with aioresponses() as mock_keenergy_api:
            mock_keenergy_api.post(
                "http://mocked-host/var/readWriteVars",
                payload=[
                    {
                        "name": "APPL.CtrlAppl.sParam.heatpump[0].OverHeatCtrl.values.tempCond",
                        "attributes": {
                            "formatId": "fmtTemp",
                            "longText": "Cond. temp.",
                            "unitId": "Temp",
                            "upperLimit": "200",
                            "lowerLimit": "0",
                        },
                        "value": "31.514103",
                    },
                ],
                headers={"Content-Type": "application/json;charset=utf-8"},
            )

            client: KebaKeEnergyAPI = KebaKeEnergyAPI(host="mocked-host")
            data: float = await client.heat_pump.get_condenser_temperature()

            assert isinstance(data, float)
            assert data == 31.51  # noqa: PLR2004

            mock_keenergy_api.assert_called_once_with(
                url="http://mocked-host/var/readWriteVars",
                data='[{"name": "APPL.CtrlAppl.sParam.heatpump[0].OverHeatCtrl.values.tempCond", "attr": "1"}]',
                method="POST",
                auth=None,
                ssl=False,
            )

    @pytest.mark.asyncio
    async def test_get_vaporizer_temperature(self) -> None:
        with aioresponses() as mock_keenergy_api:
            mock_keenergy_api.post(
                "http://mocked-host/var/readWriteVars",
                payload=[
                    {
                        "name": "APPL.CtrlAppl.sParam.heatpump[0].OverHeatCtrl.values.tempVap",
                        "attributes": {
                            "formatId": "fmtTemp",
                            "longText": "Evap. temp.",
                            "unitId": "Temp",
                        },
                        "value": "-4.5617409",
                    },
                ],
                headers={"Content-Type": "application/json;charset=utf-8"},
            )

            client: KebaKeEnergyAPI = KebaKeEnergyAPI(host="mocked-host")
            data: float = await client.heat_pump.get_vaporizer_temperature()

            assert isinstance(data, float)
            assert data == -4.56  # noqa: PLR2004

            mock_keenergy_api.assert_called_once_with(
                url="http://mocked-host/var/readWriteVars",
                data='[{"name": "APPL.CtrlAppl.sParam.heatpump[0].OverHeatCtrl.values.tempVap", "attr": "1"}]',
                method="POST",
                auth=None,
                ssl=False,
            )

    @pytest.mark.asyncio
    async def test_get_target_overheating(self) -> None:
        with aioresponses() as mock_keenergy_api:
            mock_keenergy_api.post(
                "http://mocked-host/var/readWriteVars",
                payload=[
                    {
                        "name": "APPL.CtrlAppl.sParam.heatpump[0].OverHeatCtrl.values.setOH",
                        "attributes": {
                            "formatId": "fmtTemp",
                            "longText": "Set suction SH",
                            "unitId": "TempRel",
                        },
                        "value": "5.4999752",
                    },
                ],
                headers={"Content-Type": "application/json;charset=utf-8"},
            )

            client: KebaKeEnergyAPI = KebaKeEnergyAPI(host="mocked-host")
            data: float = await client.heat_pump.get_target_overheating()

            assert isinstance(data, float)
            assert data == 5.50  # noqa: PLR2004

            mock_keenergy_api.assert_called_once_with(
                url="http://mocked-host/var/readWriteVars",
                data='[{"name": "APPL.CtrlAppl.sParam.heatpump[0].OverHeatCtrl.values.setOH", "attr": "1"}]',
                method="POST",
                auth=None,
                ssl=False,
            )

    @pytest.mark.asyncio
    async def test_get_current_overheating(self) -> None:
        with aioresponses() as mock_keenergy_api:
            mock_keenergy_api.post(
                "http://mocked-host/var/readWriteVars",
                payload=[
                    {
                        "name": "APPL.CtrlAppl.sParam.heatpump[0].OverHeatCtrl.values.actOH",
                        "attributes": {
                            "formatId": "fmtTemp",
                            "longText": "Act. suction SH",
                            "unitId": "TempRel",
                        },
                        "value": "18.676249",
                    },
                ],
                headers={"Content-Type": "application/json;charset=utf-8"},
            )

            client: KebaKeEnergyAPI = KebaKeEnergyAPI(host="mocked-host")
            data: float = await client.heat_pump.get_current_overheating()

            assert isinstance(data, float)
            assert data == 18.68  # noqa: PLR2004

            mock_keenergy_api.assert_called_once_with(
                url="http://mocked-host/var/readWriteVars",
                data='[{"name": "APPL.CtrlAppl.sParam.heatpump[0].OverHeatCtrl.values.actOH", "attr": "1"}]',
                method="POST",
                auth=None,
                ssl=False,
            )

    @pytest.mark.asyncio
    async def test_get_expansion_valve_position(self) -> None:
        with aioresponses() as mock_keenergy_api:
            mock_keenergy_api.post(
                "http://mocked-host/var/readWriteVars",
                payload=[
                    {
                        "name": "APPL.CtrlAppl.sParam.heatpump[0].OverHeatCtrl.values.stepperPos",
                        "attributes": {
                            "formatId": "fmt4p0",
                            "longText": "Stepper position",
                        },
                        "value": "20",
                    },
                ],
                headers={"Content-Type": "application/json;charset=utf-8"},
            )

            client: KebaKeEnergyAPI = KebaKeEnergyAPI(host="mocked-host")
            data: int = await client.heat_pump.get_expansion_valve_position()

            assert isinstance(data, int)
            assert data == 20  # noqa: PLR2004

            mock_keenergy_api.assert_called_once_with(
                url="http://mocked-host/var/readWriteVars",
                data='[{"name": "APPL.CtrlAppl.sParam.heatpump[0].OverHeatCtrl.values.stepperPos", "attr": "1"}]',
                method="POST",
                auth=None,
                ssl=False,
            )

    @pytest.mark.asyncio
    async def test_get_high_pressure(self) -> None:
        with aioresponses() as mock_keenergy_api:
            mock_keenergy_api.post(
                "http://mocked-host/var/readWriteVars",
                payload=[
                    {
                        "name": "APPL.CtrlAppl.sParam.heatpump[0].HighPressure.values.actValue",
                        "attributes": {
                            "formatId": "fmt3p2",
                            "longText": "High pressure",
                            "unitId": "PressBar",
                        },
                        "value": "15.018749",
                    },
                ],
                headers={"Content-Type": "application/json;charset=utf-8"},
            )

            client: KebaKeEnergyAPI = KebaKeEnergyAPI(host="mocked-host")
            data: float = await client.heat_pump.get_high_pressure()

            assert isinstance(data, float)
            assert data == 15.02  # noqa: PLR2004

            mock_keenergy_api.assert_called_once_with(
                url="http://mocked-host/var/readWriteVars",
                data='[{"name": "APPL.CtrlAppl.sParam.heatpump[0].HighPressure.values.actValue", "attr": "1"}]',
                method="POST",
                auth=None,
                ssl=False,
            )

    @pytest.mark.asyncio
    async def test_get_low_pressure(self) -> None:
        with aioresponses() as mock_keenergy_api:
            mock_keenergy_api.post(
                "http://mocked-host/var/readWriteVars",
                payload=[
                    {
                        "name": "APPL.CtrlAppl.sParam.heatpump[0].LowPressure.values.actValue",
                        "attributes": {
                            "formatId": "fmt3p2",
                            "longText": "Low pressure",
                            "unitId": "PressBar",
                        },
                        "value": "14.8125",
                    },
                ],
                headers={"Content-Type": "application/json;charset=utf-8"},
            )

            client: KebaKeEnergyAPI = KebaKeEnergyAPI(host="mocked-host")
            data: float = await client.heat_pump.get_low_pressure()

            assert isinstance(data, float)
            assert data == 14.81  # noqa: PLR2004

            mock_keenergy_api.assert_called_once_with(
                url="http://mocked-host/var/readWriteVars",
                data='[{"name": "APPL.CtrlAppl.sParam.heatpump[0].LowPressure.values.actValue", "attr": "1"}]',
                method="POST",
                auth=None,
                ssl=False,
            )

    @pytest.mark.asyncio
    @pytest.mark.parametrize(
        ("human_readable", "payload_value", "expected_value"),
        [
            (True, "true", "on"),
            (False, HeatPumpHeatRequest.ON.value, 1),
            (True, "false", "off"),
            (False, HeatPumpHeatRequest.OFF.value, 0),
        ],
    )
    async def test_get_heat_request(
        self,
        human_readable: bool,  # noqa: FBT001
        payload_value: str,
        expected_value: str,
    ) -> None:
        with aioresponses() as mock_keenergy_api:
            mock_keenergy_api.post(
                "http://mocked-host/var/readWriteVars",
                payload=[
                    {
                        "name": "APPL.CtrlAppl.sParam.heatpump[0].values.request",
                        "attributes": {},
                        "value": payload_value,
                    },
                ],
                headers={"Content-Type": "application/json;charset=utf-8"},
            )

            client: KebaKeEnergyAPI = KebaKeEnergyAPI(host="mocked-host")
            data: int | str = await client.heat_pump.get_heat_request(human_readable=human_readable)

            assert isinstance(data, (int | str))
            assert data == expected_value

            mock_keenergy_api.assert_called_once_with(
                url="http://mocked-host/var/readWriteVars",
                data='[{"name": "APPL.CtrlAppl.sParam.heatpump[0].values.request", "attr": "1"}]',
                method="POST",
                auth=None,
                ssl=False,
            )

    @pytest.mark.asyncio
    async def test_get_compressor_power(self) -> None:
        with aioresponses() as mock_keenergy_api:
            mock_keenergy_api.post(
                "http://mocked-host/var/readWriteVars",
                payload=[
                    {
                        "name": "APPL.CtrlAppl.sParam.heatpump[0].ElectricEnergyMeter.values.power",
                        "attributes": {
                            "formatId": "fmt3p2",
                            "longText": "Power input",
                            "unitId": "Pwr",
                        },
                        "value": "5.52",
                    },
                ],
                headers={"Content-Type": "application/json;charset=utf-8"},
            )

            client: KebaKeEnergyAPI = KebaKeEnergyAPI(host="mocked-host")
            data: float = await client.heat_pump.get_compressor_power()

            assert isinstance(data, float)
            assert data == 5.52  # noqa: PLR2004

            mock_keenergy_api.assert_called_once_with(
                url="http://mocked-host/var/readWriteVars",
                data='[{"name": "APPL.CtrlAppl.sParam.heatpump[0].ElectricEnergyMeter.values.power", "attr": "1"}]',
                method="POST",
                auth=None,
                ssl=False,
            )

    @pytest.mark.asyncio
    async def test_get_heating_power(self) -> None:
        with aioresponses() as mock_keenergy_api:
            mock_keenergy_api.post(
                "http://mocked-host/var/readWriteVars",
                payload=[
                    {
                        "name": "APPL.CtrlAppl.sParam.heatpump[0].HeatMeter.values.power",
                        "attributes": {
                            "formatId": "fmt3p2",
                            "longText": "Heating power",
                            "unitId": "Pwr",
                        },
                        "value": "3.22",
                    },
                ],
                headers={"Content-Type": "application/json;charset=utf-8"},
            )

            client: KebaKeEnergyAPI = KebaKeEnergyAPI(host="mocked-host")
            data: float = await client.heat_pump.get_heating_power()

            assert isinstance(data, float)
            assert data == 3.22  # noqa: PLR2004

            mock_keenergy_api.assert_called_once_with(
                url="http://mocked-host/var/readWriteVars",
                data='[{"name": "APPL.CtrlAppl.sParam.heatpump[0].HeatMeter.values.power", "attr": "1"}]',
                method="POST",
                auth=None,
                ssl=False,
            )

    @pytest.mark.asyncio
    async def test_get_hot_water_power(self) -> None:
        with aioresponses() as mock_keenergy_api:
            mock_keenergy_api.post(
                "http://mocked-host/var/readWriteVars",
                payload=[
                    {
                        "name": "APPL.CtrlAppl.sParam.heatpump[0].HotWaterMeter.values.power",
                        "attributes": {
                            "formatId": "fmt3p2",
                            "longText": "Hot water power",
                            "unitId": "Pwr",
                        },
                        "value": "2.77",
                    },
                ],
                headers={"Content-Type": "application/json;charset=utf-8"},
            )

            client: KebaKeEnergyAPI = KebaKeEnergyAPI(host="mocked-host")
            data: float = await client.heat_pump.get_hot_water_power()

            assert isinstance(data, float)
            assert data == 2.77  # noqa: PLR2004

            mock_keenergy_api.assert_called_once_with(
                url="http://mocked-host/var/readWriteVars",
                data='[{"name": "APPL.CtrlAppl.sParam.heatpump[0].HotWaterMeter.values.power", "attr": "1"}]',
                method="POST",
                auth=None,
                ssl=False,
            )

    @pytest.mark.asyncio
    async def test_get_cop(self) -> None:
        with aioresponses() as mock_keenergy_api:
            mock_keenergy_api.post(
                "http://mocked-host/var/readWriteVars",
                payload=[
                    {
                        "name": "APPL.CtrlAppl.sParam.heatpump[0].values.COP",
                        "attributes": {
                            "formatId": "fmt3p2",
                            "longText": "COP",
                        },
                        "value": "2.55",
                    },
                ],
                headers={"Content-Type": "application/json;charset=utf-8"},
            )

            client: KebaKeEnergyAPI = KebaKeEnergyAPI(host="mocked-host")
            data: float = await client.heat_pump.get_cop()

            assert isinstance(data, float)
            assert data == 2.55  # noqa: PLR2004

            mock_keenergy_api.assert_called_once_with(
                url="http://mocked-host/var/readWriteVars",
                data='[{"name": "APPL.CtrlAppl.sParam.heatpump[0].values.COP", "attr": "1"}]',
                method="POST",
                auth=None,
                ssl=False,
            )

    @pytest.mark.asyncio
    async def test_get_heating_energy(self) -> None:
        with aioresponses() as mock_keenergy_api:
            mock_keenergy_api.post(
                "http://mocked-host/var/readWriteVars",
                payload=[
                    {
                        "name": "APPL.CtrlAppl.sStatisticalData.heatpump[0].consumption.heating.energy",
                        "attributes": {
                            "formatId": "fmt4p0",
                            "longText": "Heating energy",
                            "unitId": "kWhwithoutConv",
                        },
                        "value": "8.43",
                    },
                ],
                headers={"Content-Type": "application/json;charset=utf-8"},
            )

            client: KebaKeEnergyAPI = KebaKeEnergyAPI(host="mocked-host")
            data: float = await client.heat_pump.get_heating_energy()

            assert isinstance(data, float)
            assert data == 8.43  # noqa: PLR2004

            mock_keenergy_api.assert_called_once_with(
                url="http://mocked-host/var/readWriteVars",
                data='[{"name": "APPL.CtrlAppl.sStatisticalData.heatpump[0].consumption.heating.energy", "attr": "1"}]',
                method="POST",
                auth=None,
                ssl=False,
            )

    @pytest.mark.asyncio
    async def test_get_heating_energy_consumption(self) -> None:
        with aioresponses() as mock_keenergy_api:
            mock_keenergy_api.post(
                "http://mocked-host/var/readWriteVars",
                payload=[
                    {
                        "name": "APPL.CtrlAppl.sStatisticalData.heatpump[0].consumption.heating.electricalenergy",
                        "attributes": {
                            "formatId": "fmt4p0",
                            "longText": "Heat el. energy",
                            "unitId": "kWhwithoutConv",
                        },
                        "value": "7.33",
                    },
                ],
                headers={"Content-Type": "application/json;charset=utf-8"},
            )

            client: KebaKeEnergyAPI = KebaKeEnergyAPI(host="mocked-host")
            data: float = await client.heat_pump.get_heating_energy_consumption()

            assert isinstance(data, float)
            assert data == 7.33  # noqa: PLR2004

            mock_keenergy_api.assert_called_once_with(
                url="http://mocked-host/var/readWriteVars",
                data='[{"name": "APPL.CtrlAppl.sStatisticalData.heatpump[0].consumption.heating.electricalenergy", "attr": "1"}]',  # noqa: E501
                method="POST",
                auth=None,
                ssl=False,
            )

    @pytest.mark.asyncio
    async def test_get_heating_spf(self) -> None:
        with aioresponses() as mock_keenergy_api:
            mock_keenergy_api.post(
                "http://mocked-host/var/readWriteVars",
                payload=[
                    {
                        "name": "APPL.CtrlAppl.sStatisticalData.heatpump[0].EnergyEfficiencyRatioHeat",
                        "attributes": {
                            "longText": "SPF heating",
                        },
                        "value": "3.32",
                    },
                ],
                headers={"Content-Type": "application/json;charset=utf-8"},
            )

            client: KebaKeEnergyAPI = KebaKeEnergyAPI(host="mocked-host")
            data: float = await client.heat_pump.get_heating_spf()

            assert isinstance(data, float)
            assert data == 3.32  # noqa: PLR2004

            mock_keenergy_api.assert_called_once_with(
                url="http://mocked-host/var/readWriteVars",
                data='[{"name": "APPL.CtrlAppl.sStatisticalData.heatpump[0].EnergyEfficiencyRatioHeat", "attr": "1"}]',
                method="POST",
                auth=None,
                ssl=False,
            )

    @pytest.mark.asyncio
    async def test_get_cooling_energy(self) -> None:
        with aioresponses() as mock_keenergy_api:
            mock_keenergy_api.post(
                "http://mocked-host/var/readWriteVars",
                payload=[
                    {
                        "name": "APPL.CtrlAppl.sStatisticalData.heatpump[0].consumption.cooling.energy",
                        "attributes": {
                            "formatId": "fmt4p0",
                            "longText": "Cooling energy",
                            "unitId": "kWhwithoutConv",
                        },
                        "value": "7.21",
                    },
                ],
                headers={"Content-Type": "application/json;charset=utf-8"},
            )

            client: KebaKeEnergyAPI = KebaKeEnergyAPI(host="mocked-host")
            data: float = await client.heat_pump.get_cooling_energy()

            assert isinstance(data, float)
            assert data == 7.21  # noqa: PLR2004

            mock_keenergy_api.assert_called_once_with(
                url="http://mocked-host/var/readWriteVars",
                data='[{"name": "APPL.CtrlAppl.sStatisticalData.heatpump[0].consumption.cooling.energy", "attr": "1"}]',
                method="POST",
                auth=None,
                ssl=False,
            )

    @pytest.mark.asyncio
    async def test_get_cooling_energy_consumption(self) -> None:
        with aioresponses() as mock_keenergy_api:
            mock_keenergy_api.post(
                "http://mocked-host/var/readWriteVars",
                payload=[
                    {
                        "name": "APPL.CtrlAppl.sStatisticalData.heatpump[0].consumption.cooling.electricalenergy",
                        "attributes": {
                            "formatId": "fmt4p0",
                            "longText": "Cool el. energy",
                            "unitId": "kWhwithoutConv",
                        },
                        "value": "8.72",
                    },
                ],
                headers={"Content-Type": "application/json;charset=utf-8"},
            )

            client: KebaKeEnergyAPI = KebaKeEnergyAPI(host="mocked-host")
            data: float = await client.heat_pump.get_cooling_energy_consumption()

            assert isinstance(data, float)
            assert data == 8.72  # noqa: PLR2004

            mock_keenergy_api.assert_called_once_with(
                url="http://mocked-host/var/readWriteVars",
                data='[{"name": "APPL.CtrlAppl.sStatisticalData.heatpump[0].consumption.cooling.electricalenergy", "attr": "1"}]',  # noqa: E501
                method="POST",
                auth=None,
                ssl=False,
            )

    @pytest.mark.asyncio
    async def test_get_cooling_spf(self) -> None:
        with aioresponses() as mock_keenergy_api:
            mock_keenergy_api.post(
                "http://mocked-host/var/readWriteVars",
                payload=[
                    {
                        "name": "APPL.CtrlAppl.sStatisticalData.heatpump[0].EnergyEfficiencyRatioCool",
                        "attributes": {
                            "formatId": "fmt3p2",
                            "longText": "SPF cooling",
                        },
                        "value": "4.22",
                    },
                ],
                headers={"Content-Type": "application/json;charset=utf-8"},
            )

            client: KebaKeEnergyAPI = KebaKeEnergyAPI(host="mocked-host")
            data: float = await client.heat_pump.get_cooling_spf()

            assert isinstance(data, float)
            assert data == 4.22  # noqa: PLR2004

            mock_keenergy_api.assert_called_once_with(
                url="http://mocked-host/var/readWriteVars",
                data='[{"name": "APPL.CtrlAppl.sStatisticalData.heatpump[0].EnergyEfficiencyRatioCool", "attr": "1"}]',
                method="POST",
                auth=None,
                ssl=False,
            )

    @pytest.mark.asyncio
    async def test_get_hot_water_energy(self) -> None:
        with aioresponses() as mock_keenergy_api:
            mock_keenergy_api.post(
                "http://mocked-host/var/readWriteVars",
                payload=[
                    {
                        "name": "APPL.CtrlAppl.sStatisticalData.heatpump[0].consumption.domHotWater.energy",
                        "attributes": {
                            "formatId": "fmt4p0",
                            "longText": "Dom. HW energy",
                            "unitId": "kWhwithoutConv",
                        },
                        "value": "7.86",
                    },
                ],
                headers={"Content-Type": "application/json;charset=utf-8"},
            )

            client: KebaKeEnergyAPI = KebaKeEnergyAPI(host="mocked-host")
            data: float = await client.heat_pump.get_hot_water_energy()

            assert isinstance(data, float)
            assert data == 7.86  # noqa: PLR2004

            mock_keenergy_api.assert_called_once_with(
                url="http://mocked-host/var/readWriteVars",
                data='[{"name": "APPL.CtrlAppl.sStatisticalData.heatpump[0].consumption.domHotWater.energy", "attr": "1"}]',  # noqa: E501
                method="POST",
                auth=None,
                ssl=False,
            )

    @pytest.mark.asyncio
    async def test_get_hot_water_energy_consumption(self) -> None:
        with aioresponses() as mock_keenergy_api:
            mock_keenergy_api.post(
                "http://mocked-host/var/readWriteVars",
                payload=[
                    {
                        "name": "APPL.CtrlAppl.sStatisticalData.heatpump[0].consumption.domHotWater.electricalenergy",
                        "attributes": {
                            "formatId": "fmt4p0",
                            "longText": "Dom.HW el. energy",
                            "unitId": "kWhwithoutConv",
                        },
                        "value": "2.77",
                    },
                ],
                headers={"Content-Type": "application/json;charset=utf-8"},
            )

            client: KebaKeEnergyAPI = KebaKeEnergyAPI(host="mocked-host")
            data: float = await client.heat_pump.get_hot_water_energy_consumption()

            assert isinstance(data, float)
            assert data == 2.77  # noqa: PLR2004

            mock_keenergy_api.assert_called_once_with(
                url="http://mocked-host/var/readWriteVars",
                data='[{"name": "APPL.CtrlAppl.sStatisticalData.heatpump[0].consumption.domHotWater.electricalenergy", "attr": "1"}]',  # noqa: E501
                method="POST",
                auth=None,
                ssl=False,
            )

    @pytest.mark.asyncio
    async def test_get_hot_water_spf(self) -> None:
        with aioresponses() as mock_keenergy_api:
            mock_keenergy_api.post(
                "http://mocked-host/var/readWriteVars",
                payload=[
                    {
                        "name": "APPL.CtrlAppl.sStatisticalData.heatpump[0].EnergyEfficiencyRatioDomHotWater",
                        "attributes": {
                            "longText": "SPF DHW",
                        },
                        "value": "2.50",
                    },
                ],
                headers={"Content-Type": "application/json;charset=utf-8"},
            )

            client: KebaKeEnergyAPI = KebaKeEnergyAPI(host="mocked-host")
            data: float = await client.heat_pump.get_hot_water_spf()

            assert isinstance(data, float)
            assert data == 2.50  # noqa: PLR2004

            mock_keenergy_api.assert_called_once_with(
                url="http://mocked-host/var/readWriteVars",
                data='[{"name": "APPL.CtrlAppl.sStatisticalData.heatpump[0].EnergyEfficiencyRatioDomHotWater", "attr": "1"}]',  # noqa: E501
                method="POST",
                auth=None,
                ssl=False,
            )

    @pytest.mark.asyncio
    async def test_get_total_thermal_energy(self) -> None:
        with aioresponses() as mock_keenergy_api:
            mock_keenergy_api.post(
                "http://mocked-host/var/readWriteVars",
                payload=[
                    {
                        "name": "APPL.CtrlAppl.sStatisticalData.heatpump[0].consumption.energy",
                        "attributes": {
                            "formatId": "fmt4p0",
                            "longText": "Energy",
                            "unitId": "kWhwithoutConv",
                        },
                        "value": "8.22",
                    },
                ],
                headers={"Content-Type": "application/json;charset=utf-8"},
            )

            client: KebaKeEnergyAPI = KebaKeEnergyAPI(host="mocked-host")
            data: float = await client.heat_pump.get_total_thermal_energy()

            assert isinstance(data, float)
            assert data == 8.22  # noqa: PLR2004

            mock_keenergy_api.assert_called_once_with(
                url="http://mocked-host/var/readWriteVars",
                data='[{"name": "APPL.CtrlAppl.sStatisticalData.heatpump[0].consumption.energy", "attr": "1"}]',
                method="POST",
                auth=None,
                ssl=False,
            )

    @pytest.mark.asyncio
    async def test_get_total_energy_consumption(self) -> None:
        with aioresponses() as mock_keenergy_api:
            mock_keenergy_api.post(
                "http://mocked-host/var/readWriteVars",
                payload=[
                    {
                        "name": "APPL.CtrlAppl.sStatisticalData.heatpump[0].consumption.electricalenergy",
                        "attributes": {
                            "formatId": "fmt4p0",
                            "longText": "El. energy",
                            "unitId": "kWhwithoutConv",
                        },
                        "value": "5.21",
                    },
                ],
                headers={"Content-Type": "application/json;charset=utf-8"},
            )

            client: KebaKeEnergyAPI = KebaKeEnergyAPI(host="mocked-host")
            data: float = await client.heat_pump.get_total_energy_consumption()

            assert isinstance(data, float)
            assert data == 5.21  # noqa: PLR2004

            mock_keenergy_api.assert_called_once_with(
                url="http://mocked-host/var/readWriteVars",
                data='[{"name": "APPL.CtrlAppl.sStatisticalData.heatpump[0].consumption.electricalenergy", "attr": "1"}]',  # noqa: E501
                method="POST",
                auth=None,
                ssl=False,
            )

    @pytest.mark.asyncio
    async def test_get_total_spf(self) -> None:
        with aioresponses() as mock_keenergy_api:
            mock_keenergy_api.post(
                "http://mocked-host/var/readWriteVars",
                payload=[
                    {
                        "name": "APPL.CtrlAppl.sStatisticalData.heatpump[0].EnergyEfficiencyRatio",
                        "attributes": {
                            "formatId": "fmt3p2",
                            "longText": "SPF general",
                        },
                        "value": "2.43",
                    },
                ],
                headers={"Content-Type": "application/json;charset=utf-8"},
            )

            client: KebaKeEnergyAPI = KebaKeEnergyAPI(host="mocked-host")
            data: float = await client.heat_pump.get_total_spf()

            assert isinstance(data, float)
            assert data == 2.43  # noqa: PLR2004

            mock_keenergy_api.assert_called_once_with(
                url="http://mocked-host/var/readWriteVars",
                data='[{"name": "APPL.CtrlAppl.sStatisticalData.heatpump[0].EnergyEfficiencyRatio", "attr": "1"}]',
                method="POST",
                auth=None,
                ssl=False,
            )

    @pytest.mark.asyncio
    @pytest.mark.parametrize(
        ("human_readable", "payload_value", "expected_value"),
        [
            (True, "true", "on"),
            (False, HeatPumpHasPassiveCooling.ON.value, 1),
            (True, "false", "off"),
            (False, HeatPumpHasPassiveCooling.OFF.value, 0),
        ],
    )
    async def test_has_passive_cooling(
        self,
        human_readable: bool,  # noqa: FBT001
        payload_value: str,
        expected_value: str,
    ) -> None:
        with aioresponses() as mock_keenergy_api:
            mock_keenergy_api.post(
                "http://mocked-host/var/readWriteVars",
                payload=[
                    {
                        "name": "APPL.CtrlAppl.sParam.options.heatpump[0].hasPassiveCooling",
                        "attributes": {"longText": "With passive cooling,"},
                        "value": payload_value,
                    },
                ],
                headers={"Content-Type": "application/json;charset=utf-8"},
            )

            client: KebaKeEnergyAPI = KebaKeEnergyAPI(host="mocked-host")
            data: int | str = await client.heat_pump.has_passive_cooling(human_readable=human_readable)

            assert isinstance(data, (int | str))
            assert data == expected_value

            mock_keenergy_api.assert_called_once_with(
                url="http://mocked-host/var/readWriteVars",
                data='[{"name": "APPL.CtrlAppl.sParam.options.heatpump[0].hasPassiveCooling", "attr": "1"}]',
                method="POST",
                auth=None,
                ssl=False,
            )

    @pytest.mark.asyncio
    async def test_get_operating_time(self) -> None:
        with aioresponses() as mock_keenergy_api:
            mock_keenergy_api.post(
                "http://mocked-host/var/readWriteVars",
                payload=[
                    {
                        "name": "APPL.CtrlAppl.sParam.heatpump[0].operationalData.operationalTimeS",
                        "attributes": {
                            "formatId": "fmt6p0",
                            "longText": "Operational hrs.",
                            "unitId": "TimeHour",
                        },
                        "value": "65037217",
                    },
                ],
                headers={"Content-Type": "application/json;charset=utf-8"},
            )

            client: KebaKeEnergyAPI = KebaKeEnergyAPI(host="mocked-host")
            data: int = await client.heat_pump.get_operating_time()

            assert isinstance(data, int)
            assert data == 65037217  # noqa: PLR2004

            mock_keenergy_api.assert_called_once_with(
                url="http://mocked-host/var/readWriteVars",
                data='[{"name": "APPL.CtrlAppl.sParam.heatpump[0].operationalData.operationalTimeS", "attr": "1"}]',
                method="POST",
                auth=None,
                ssl=False,
            )

    @pytest.mark.asyncio
    async def test_get_max_runtime(self) -> None:
        with aioresponses() as mock_keenergy_api:
            mock_keenergy_api.post(
                "http://mocked-host/var/readWriteVars",
                payload=[
                    {
                        "name": "APPL.CtrlAppl.sParam.heatpump[0].operationalData.maxRunTimeS",
                        "attributes": {
                            "formatId": "fmt6p1",
                            "longText": "Max run-time",
                            "unitId": "TimeHour",
                        },
                        "value": "8836597",
                    },
                ],
                headers={"Content-Type": "application/json;charset=utf-8"},
            )

            client: KebaKeEnergyAPI = KebaKeEnergyAPI(host="mocked-host")
            data: int = await client.heat_pump.get_max_runtime()

            assert isinstance(data, int)
            assert data == 8836597  # noqa: PLR2004

            mock_keenergy_api.assert_called_once_with(
                url="http://mocked-host/var/readWriteVars",
                data='[{"name": "APPL.CtrlAppl.sParam.heatpump[0].operationalData.maxRunTimeS", "attr": "1"}]',
                method="POST",
                auth=None,
                ssl=False,
            )

    @pytest.mark.asyncio
    async def test_get_activation_counter(self) -> None:
        with aioresponses() as mock_keenergy_api:
            mock_keenergy_api.post(
                "http://mocked-host/var/readWriteVars",
                payload=[
                    {
                        "name": "APPL.CtrlAppl.sParam.heatpump[0].operationalData.activationCounter",
                        "attributes": {
                            "formatId": "fmt6p0",
                            "longText": "Turn-on cycles",
                        },
                        "value": "1197",
                    },
                ],
                headers={"Content-Type": "application/json;charset=utf-8"},
            )

            client: KebaKeEnergyAPI = KebaKeEnergyAPI(host="mocked-host")
            data: int = await client.heat_pump.get_activation_counter()

            assert isinstance(data, int)
            assert data == 1197  # noqa: PLR2004

            mock_keenergy_api.assert_called_once_with(
                url="http://mocked-host/var/readWriteVars",
                data='[{"name": "APPL.CtrlAppl.sParam.heatpump[0].operationalData.activationCounter", "attr": "1"}]',
                method="POST",
                auth=None,
                ssl=False,
            )

    @pytest.mark.asyncio
    @pytest.mark.parametrize(
        ("human_readable", "payload_value", "expected_value"),
        [
            (True, "true", "on"),
            (False, HeatPumpHasCompressorFailure.ON.value, 1),
            (True, "false", "off"),
            (False, HeatPumpHasCompressorFailure.OFF.value, 0),
        ],
    )
    async def test_has_compressor_failure(
        self,
        human_readable: bool,  # noqa: FBT001
        payload_value: str,
        expected_value: str,
    ) -> None:
        with aioresponses() as mock_keenergy_api:
            mock_keenergy_api.post(
                "http://mocked-host/var/readWriteVars",
                payload=[
                    {
                        "name": "APPL.CtrlAppl.sParam.heatpump[0].FailureCompressor.values.actValue",
                        "attributes": {
                            "longText": "Failure compressor",
                        },
                        "value": payload_value,
                    },
                ],
                headers={"Content-Type": "application/json;charset=utf-8"},
            )

            client: KebaKeEnergyAPI = KebaKeEnergyAPI(host="mocked-host")
            data: int | str = await client.heat_pump.has_compressor_failure(human_readable=human_readable)

            assert isinstance(data, (int | str))
            assert data == expected_value

            mock_keenergy_api.assert_called_once_with(
                url="http://mocked-host/var/readWriteVars",
                data='[{"name": "APPL.CtrlAppl.sParam.heatpump[0].FailureCompressor.values.actValue", "attr": "1"}]',
                method="POST",
                auth=None,
                ssl=False,
            )

    @pytest.mark.asyncio
    @pytest.mark.parametrize(
        ("human_readable", "payload_value", "expected_value"),
        [
            (True, "true", "on"),
            (False, HeatPumpHasSourceFailure.ON.value, 1),
            (True, "false", "off"),
            (False, HeatPumpHasSourceFailure.OFF.value, 0),
        ],
    )
    async def test_has_source_failure(
        self,
        human_readable: bool,  # noqa: FBT001
        payload_value: str,
        expected_value: str,
    ) -> None:
        with aioresponses() as mock_keenergy_api:
            mock_keenergy_api.post(
                "http://mocked-host/var/readWriteVars",
                payload=[
                    {
                        "name": "APPL.CtrlAppl.sParam.heatpump[0].FailureSource.values.actValue",
                        "attributes": {
                            "longText": "Failure Source",
                        },
                        "value": payload_value,
                    },
                ],
                headers={"Content-Type": "application/json;charset=utf-8"},
            )

            client: KebaKeEnergyAPI = KebaKeEnergyAPI(host="mocked-host")
            data: int | str = await client.heat_pump.has_source_failure(human_readable=human_readable)

            assert isinstance(data, (int | str))
            assert data == expected_value

            mock_keenergy_api.assert_called_once_with(
                url="http://mocked-host/var/readWriteVars",
                data='[{"name": "APPL.CtrlAppl.sParam.heatpump[0].FailureSource.values.actValue", "attr": "1"}]',
                method="POST",
                auth=None,
                ssl=False,
            )

    @pytest.mark.asyncio
    @pytest.mark.parametrize(
        ("human_readable", "payload_value", "expected_value"),
        [
            (True, "true", "on"),
            (False, HeatPumpHasSourceActuatorFailure.ON.value, 1),
            (True, "false", "off"),
            (False, HeatPumpHasSourceActuatorFailure.OFF.value, 0),
        ],
    )
    async def test_has_source_actuator_failure(
        self,
        human_readable: bool,  # noqa: FBT001
        payload_value: str,
        expected_value: str,
    ) -> None:
        with aioresponses() as mock_keenergy_api:
            mock_keenergy_api.post(
                "http://mocked-host/var/readWriteVars",
                payload=[
                    {
                        "name": "APPL.CtrlAppl.sParam.heatpump[0].FailureActuatorSource.values.actValue",
                        "attributes": {
                            "longText": "Failure actuator src",
                        },
                        "value": payload_value,
                    },
                ],
                headers={"Content-Type": "application/json;charset=utf-8"},
            )

            client: KebaKeEnergyAPI = KebaKeEnergyAPI(host="mocked-host")
            data: int | str = await client.heat_pump.has_source_actuator_failure(human_readable=human_readable)

            assert isinstance(data, (int | str))
            assert data == expected_value

            mock_keenergy_api.assert_called_once_with(
                url="http://mocked-host/var/readWriteVars",
                data=(
                    '[{"name": "APPL.CtrlAppl.sParam.heatpump[0].FailureActuatorSource.values.actValue", "attr": "1"}]'
                ),
                method="POST",
                auth=None,
                ssl=False,
            )

    @pytest.mark.asyncio
    @pytest.mark.parametrize(
        ("human_readable", "payload_value", "expected_value"),
        [
            (True, "true", "on"),
            (False, HeatPumpHasThreePhaseFailure.ON.value, 1),
            (True, "false", "off"),
            (False, HeatPumpHasThreePhaseFailure.OFF.value, 0),
        ],
    )
    async def test_has_three_phase_failure(
        self,
        human_readable: bool,  # noqa: FBT001
        payload_value: str,
        expected_value: str,
    ) -> None:
        with aioresponses() as mock_keenergy_api:
            mock_keenergy_api.post(
                "http://mocked-host/var/readWriteVars",
                payload=[
                    {
                        "name": "APPL.CtrlAppl.sParam.heatpump[0].FailureThreePhase.values.actValue",
                        "attributes": {
                            "longText": "Failure 3-Phase",
                        },
                        "value": payload_value,
                    },
                ],
                headers={"Content-Type": "application/json;charset=utf-8"},
            )

            client: KebaKeEnergyAPI = KebaKeEnergyAPI(host="mocked-host")
            data: int | str = await client.heat_pump.has_three_phase_failure(human_readable=human_readable)

            assert isinstance(data, (int | str))
            assert data == expected_value

            mock_keenergy_api.assert_called_once_with(
                url="http://mocked-host/var/readWriteVars",
                data='[{"name": "APPL.CtrlAppl.sParam.heatpump[0].FailureThreePhase.values.actValue", "attr": "1"}]',
                method="POST",
                auth=None,
                ssl=False,
            )

    @pytest.mark.asyncio
    @pytest.mark.parametrize(
        ("human_readable", "payload_value", "expected_value"),
        [
            (True, "true", "on"),
            (False, HeatPumpHasSourcePressureFailure.ON.value, 1),
            (True, "false", "off"),
            (False, HeatPumpHasSourcePressureFailure.OFF.value, 0),
        ],
    )
    async def test_has_source_pressure_failure(
        self,
        human_readable: bool,  # noqa: FBT001
        payload_value: str,
        expected_value: str,
    ) -> None:
        with aioresponses() as mock_keenergy_api:
            mock_keenergy_api.post(
                "http://mocked-host/var/readWriteVars",
                payload=[
                    {
                        "name": "APPL.CtrlAppl.sParam.heatpump[0].FailureSrcPressure.values.actValue",
                        "attributes": {
                            "longText": "Failure src pressure",
                        },
                        "value": payload_value,
                    },
                ],
                headers={"Content-Type": "application/json;charset=utf-8"},
            )

            client: KebaKeEnergyAPI = KebaKeEnergyAPI(host="mocked-host")
            data: int | str = await client.heat_pump.has_source_pressure_failure(human_readable=human_readable)

            assert isinstance(data, (int | str))
            assert data == expected_value

            mock_keenergy_api.assert_called_once_with(
                url="http://mocked-host/var/readWriteVars",
                data='[{"name": "APPL.CtrlAppl.sParam.heatpump[0].FailureSrcPressure.values.actValue", "attr": "1"}]',
                method="POST",
                auth=None,
                ssl=False,
            )

    @pytest.mark.asyncio
    @pytest.mark.parametrize(
        ("human_readable", "payload_value", "expected_value"),
        [
            (True, "true", "on"),
            (False, HeatPumpHasVFDFailure.ON.value, 1),
            (True, "false", "off"),
            (False, HeatPumpHasVFDFailure.OFF.value, 0),
        ],
    )
    async def test_has_vfd_failure(
        self,
        human_readable: bool,  # noqa: FBT001
        payload_value: str,
        expected_value: str,
    ) -> None:
        with aioresponses() as mock_keenergy_api:
            mock_keenergy_api.post(
                "http://mocked-host/var/readWriteVars",
                payload=[
                    {
                        "name": "APPL.CtrlAppl.sParam.heatpump[0].FailureVFD.values.actValue",
                        "attributes": {
                            "longText": "Failure VFD",
                        },
                        "value": payload_value,
                    },
                ],
                headers={"Content-Type": "application/json;charset=utf-8"},
            )

            client: KebaKeEnergyAPI = KebaKeEnergyAPI(host="mocked-host")
            data: int | str = await client.heat_pump.has_vfd_failure(human_readable=human_readable)

            assert isinstance(data, (int | str))
            assert data == expected_value

            mock_keenergy_api.assert_called_once_with(
                url="http://mocked-host/var/readWriteVars",
                data='[{"name": "APPL.CtrlAppl.sParam.heatpump[0].FailureVFD.values.actValue", "attr": "1"}]',
                method="POST",
                auth=None,
                ssl=False,
            )


@pytest.mark.unhappy
class TestUnhappyPathHeatPumpSection:

    @pytest.mark.asyncio
    @pytest.mark.parametrize(
        "operating_mode",
        ["INVALID"],
    )
    async def test_set_invalid_operating_mode(
        self,
        operating_mode: int | str,
    ) -> None:
        with aioresponses() as mock_keenergy_api:
            mock_keenergy_api.post(
                "http://mocked-host/var/readWriteVars?action=set",
                payload={},
                headers={"Content-Type": "application/json;charset=utf-8"},
            )

            client: KebaKeEnergyAPI = KebaKeEnergyAPI(host="mocked-host")

            with pytest.raises(APIError) as error:
                await client.heat_pump.set_operating_mode(operating_mode)

            assert str(error.value) == "Invalid value! Allowed values are ['OFF', '0', 'ON', '1', 'BACKUP', '2']"

            mock_keenergy_api.assert_not_called()

    @pytest.mark.asyncio
    @pytest.mark.parametrize(
        "operating_mode",
        ["INVALID"],
    )
    async def test_set_invalid_compressor_use_night_speed(
        self,
        operating_mode: int | str,
    ) -> None:
        with aioresponses() as mock_keenergy_api:
            mock_keenergy_api.post(
                "http://mocked-host/var/readWriteVars?action=set",
                payload={},
                headers={"Content-Type": "application/json;charset=utf-8"},
            )

            client: KebaKeEnergyAPI = KebaKeEnergyAPI(host="mocked-host")

            with pytest.raises(APIError, match=r"Invalid value! Allowed values are \['OFF', '0', 'ON', '1']"):
                await client.heat_pump.set_compressor_use_night_speed(operating_mode)

            mock_keenergy_api.assert_not_called()



"""KEBA KeEnergy API error classes."""

from http import HTTPStatus


class APIError(Exception):
    """API error."""

    def __init__(
        self,
        message: str = "",
        /,
        *,
        status: HTTPStatus | None = None,
    ) -> None:
        _message: str = message

        if status:
            _message = f"{status} {status.phrase}: {status.description}"

            if message:
                _message = f"{_message} - {message}"

        self.message: str = _message
        self.status: int | None = status

    def __str__(self) -> str:
        return self.message


class InvalidJsonError(APIError):
    """Invalid JSON data error."""


class AuthenticationError(APIError):
    """Invalid credentials error."""



from typing import Any

from tests.test_endpoints.test_heat_circuit_section_data import heating_curve_points_expected_data
from tests.test_endpoints.test_heat_circuit_section_data import heating_curve_points_payload

read_data_payload_1: list[dict[str, Any]] = [
    {
        "name": "APPL.CtrlAppl.sParam.options.systemNumberOfHotWaterTanks",
        "attributes": {
            "dynLowerLimit": 1,
            "dynUpperLimit": 1,
            "formatId": "fmt2p0",
            "longText": "Qty heat pumps",
            "lowerLimit": "0",
            "upperLimit": "4",
        },
        "value": "2",
    },
    {
        "name": "APPL.CtrlAppl.sParam.hotWaterTank[0].topTemp.values.actValue",
        "attributes": {
            "formatId": "fmtTemp",
            "longText": "Temp. act.",
            "lowerLimit": "20",
            "unitId": "Temp",
            "upperLimit": "90",
        },
        "value": "40.808357",
    },
    {
        "name": "APPL.CtrlAppl.sParam.extHeatSource[0].values.setTemp",
        "attributes": {
            "formatId": "fmtTemp",
            "longText": "Temp. nom.",
            "unitId": "Temp",
            "upperLimit": "90",
            "lowerLimit": "20",
        },
        "value": "22.56",
    },
    {
        "name": "APPL.CtrlAppl.sParam.photovoltaics.ElectricEnergyMeter.values.accumulatedHeat",
        "attributes": {
            "formatId": "fmt6p0",
            "longText": "Acc. energy",
            "unitId": "kWh",
        },
        "value": "349442.23",
    },
]

read_data_expected_data_1: str = (
    '[{"name": "APPL.CtrlAppl.sParam.options.systemNumberOfHotWaterTanks", "attr": "1"}, '
    '{"name": "APPL.CtrlAppl.sParam.hotWaterTank[0].topTemp.values.actValue", "attr": "1"}, '
    '{"name": "APPL.CtrlAppl.sParam.extHeatSource[0].values.setTemp", "attr": "1"}, '
    '{"name": "APPL.CtrlAppl.sParam.photovoltaics.ElectricEnergyMeter.values.accumulatedHeat", "attr": "1"}]'
)

read_data_expected_response_1: dict[str, Any] = {
    "system": {
        "hot_water_tank_numbers": {"value": 2, "attributes": {"lower_limit": "0", "upper_limit": "4"}},
    },
    "buffer_tank": {},
    "hot_water_tank": {
        "current_temperature": [
            {
                "attributes": {"lower_limit": "20", "upper_limit": "90"},
                "value": 40.81,
            },
        ],
    },
    "heat_pump": {},
    "heat_circuit": {},
    "solar_circuit": {},
    "switch_valve": {},
    "external_heat_source": {
        "target_temperature": [
            {
                "attributes": {"lower_limit": "20", "upper_limit": "90"},
                "value": 22.56,
            },
        ],
    },
    "photovoltaic": {
        "total_energy": {
            "attributes": {},
            "value": 349442.23,
        },
    },
}

read_data_payload_2: list[dict[str, Any]] = [
    {
        "name": "APPL.CtrlAppl.sParam.heatCircuit[0].values.setValue",
        "attributes": {
            "formatId": "fmtTemp",
            "longText": "Room temp. Nom.",
            "lowerLimit": "10",
            "unitId": "Temp",
            "upperLimit": "90",
        },
        "value": "10.808357",
    },
    {
        "name": "APPL.CtrlAppl.sParam.heatCircuit[2].values.setValue",
        "attributes": {
            "formatId": "fmtTemp",
            "longText": "Room temp. Nom.",
            "lowerLimit": "10",
            "unitId": "Temp",
            "upperLimit": "90",
        },
        "value": "11.808357",
    },
    {
        "name": "APPL.CtrlAppl.sParam.heatpump[0].TempHeatFlow.values.actValue",
        "attributes": {
            "formatId": "fmtTemp",
            "longText": "Inflow temp.",
            "unitId": "Temp",
        },
        "value": "24.200001",
    },
    {
        "name": "APPL.CtrlAppl.sParam.heatpump[2].TempHeatFlow.values.actValue",
        "attributes": {
            "formatId": "fmtTemp",
            "longText": "Inflow temp.",
            "unitId": "Temp",
        },
        "value": "23.200001",
    },
]

read_data_expected_data_2: str = (
    '[{"name": "APPL.CtrlAppl.sParam.heatCircuit[0].values.setValue", "attr": "1"}, '
    '{"name": "APPL.CtrlAppl.sParam.heatCircuit[2].values.setValue", "attr": "1"}, '
    '{"name": "APPL.CtrlAppl.sParam.heatpump[0].TempHeatFlow.values.actValue", "attr": "1"}, '
    '{"name": "APPL.CtrlAppl.sParam.heatpump[2].TempHeatFlow.values.actValue", "attr": "1"}]'
)

read_data_expected_response_2: dict[str, Any] = {
    "system": {},
    "buffer_tank": {},
    "hot_water_tank": {},
    "heat_pump": {
        "flow_temperature": [{"value": 24.2, "attributes": {}}, {"value": 23.2, "attributes": {}}],
    },
    "heat_circuit": {
        "target_temperature": [
            {"value": 10.81, "attributes": {"lower_limit": "10", "upper_limit": "90"}},
            {"value": 11.81, "attributes": {"lower_limit": "10", "upper_limit": "90"}},
        ],
    },
    "solar_circuit": {},
    "switch_valve": {},
    "external_heat_source": {},
    "photovoltaic": {},
}

read_data_option_payload_3: list[dict[str, Any]] = [
    {
        "name": "APPL.CtrlAppl.sParam.options.systemNumberOfHeatPumps",
        "attributes": {
            "dynLowerLimit": 1,
            "dynUpperLimit": 1,
            "formatId": "fmt2p0",
            "longText": "Qty heat pumps",
            "lowerLimit": "0",
            "upperLimit": "4",
        },
        "value": "2",
    },
    {
        "name": "APPL.CtrlAppl.sParam.options.systemNumberOfHeatingCircuits",
        "attributes": {
            "dynLowerLimit": 1,
            "dynUpperLimit": 1,
            "formatId": "fmt2p0",
            "longText": "Qty HC",
            "lowerLimit": "0",
            "upperLimit": "8",
        },
        "value": "1",
    },
    {
        "name": "APPL.CtrlAppl.sParam.options.systemNumberOfSolarCircuits",
        "attributes": {
            "dynLowerLimit": 1,
            "dynUpperLimit": 1,
            "formatId": "fmt2p0",
            "longText": "Qty solar",
            "upperLimit": "4",
            "lowerLimit": "0",
        },
        "value": "1",
    },
    {
        "name": "APPL.CtrlAppl.sParam.options.systemNumberOfBuffers",
        "attributes": {
            "formatId": "fmt2p0",
            "longText": "Qty buffers",
            "upperLimit": "0",
            "lowerLimit": "0",
            "dynLowerLimit": 1,
            "dynUpperLimit": 1,
        },
        "value": "0",
    },
    {
        "name": "APPL.CtrlAppl.sParam.options.systemNumberOfHotWaterTanks",
        "attributes": {
            "dynLowerLimit": 1,
            "dynUpperLimit": 1,
            "formatId": "fmt2p0",
            "longText": "Qty HW tank",
            "lowerLimit": "0",
            "upperLimit": "4",
        },
        "value": "1",
    },
    {
        "name": "APPL.CtrlAppl.sParam.options.systemNumberOfExtHeatSources",
        "attributes": {
            "formatId": "fmt2p0",
            "longText": "Qty ext. heat sources",
            "upperLimit": "1",
            "lowerLimit": "0",
        },
        "value": "0",
    },
    {
        "name": "APPL.CtrlAppl.sParam.options.systemNumberOfSwitchValves",
        "attributes": {
            "formatId": "fmt2p0",
            "longText": "Qty switch valves",
            "upperLimit": "0",
            "lowerLimit": "0",
            "dynLowerLimit": 1,
            "dynUpperLimit": 1,
        },
        "value": "1",
    },
]

read_data_payload_3: list[dict[str, Any]] = [
    {
        "name": "APPL.CtrlAppl.sParam.outdoorTemp.values.actValue",
        "attributes": {
            "formatId": "fmtTemp",
            "longText": "Exterior temp.",
            "lowerLimit": "-100",
            "unitId": "Temp",
            "upperLimit": "100",
        },
        "value": "17.54",
    },
    {
        "name": "APPL.CtrlAppl.sParam.heatCircuit[0].values.setValue",
        "attributes": {
            "formatId": "fmtTemp",
            "longText": "Room temp. Nom.",
            "lowerLimit": "10",
            "unitId": "Temp",
            "upperLimit": "90",
        },
        "value": "10.808357",
    },
    {
        "name": "APPL.CtrlAppl.sParam.heatCircuit[0].param.linTab.fileName",
        "attributes": {
            "longText": "Heat curve",
        },
        "value": "HC1",
    },
    {
        "name": "APPL.CtrlAppl.sParam.heatpump[0].TempHeatFlow.values.actValue",
        "attributes": {
            "formatId": "fmtTemp",
            "longText": "Inflow temp.",
            "unitId": "Temp",
        },
        "value": "24.200001",
    },
    {
        "name": "APPL.CtrlAppl.sParam.heatpump[1].TempHeatFlow.values.actValue",
        "attributes": {
            "formatId": "fmtTemp",
            "longText": "Inflow temp.",
            "unitId": "Temp",
        },
        "value": "23.2",
    },
    {
        "name": "APPL.CtrlAppl.sParam.switchvalve[0].values.actPosition",
        "attributes": {
            "formatId": "fmtSwitchValveStateV1",
            "longText": "Valve pos.",
            "unitId": "Enum",
            "upperLimit": "2",
            "lowerLimit": "0",
        },
        "value": "1",
    },
]

read_data_extra_attributes_payload_3: list[dict[str, Any]] = heating_curve_points_payload

read_data_expected_data_3: str = (
    '[{"name": "APPL.CtrlAppl.sParam.outdoorTemp.values.actValue", "attr": "1"}, '
    '{"name": "APPL.CtrlAppl.sParam.heatCircuit[0].values.setValue", "attr": "1"}, '
    '{"name": "APPL.CtrlAppl.sParam.heatCircuit[0].param.linTab.fileName", "attr": "1"}, '
    '{"name": "APPL.CtrlAppl.sParam.heatpump[0].TempHeatFlow.values.actValue", "attr": "1"}, '
    '{"name": "APPL.CtrlAppl.sParam.heatpump[1].TempHeatFlow.values.actValue", "attr": "1"}, '
    '{"name": "APPL.CtrlAppl.sParam.switchvalve[0].values.actPosition", "attr": "1"}]'
)

read_data_expected_response_3: dict[str, Any] = {
    "system": {
        "outdoor_temperature": {
            "value": 17.54,
            "attributes": {"lower_limit": "-100", "upper_limit": "100"},
        },
    },
    "buffer_tank": {},
    "hot_water_tank": {},
    "heat_pump": {
        "flow_temperature": [
            {
                "value": 24.2,
                "attributes": {},
            },
            {
                "value": 23.2,
                "attributes": {},
            },
        ],
    },
    "heat_circuit": {
        "heating_curve": [
            {
                "attributes": {
                    "points": [
                        {
                            "flow": 20.0,
                            "outdoor": -15.0,
                        },
                        {
                            "flow": 20.0,
                            "outdoor": -15.0,
                        },
                        {
                            "flow": 20.0,
                            "outdoor": -15.0,
                        },
                        {
                            "flow": 20.0,
                            "outdoor": -15.0,
                        },
                        {
                            "flow": 20.0,
                            "outdoor": -15.0,
                        },
                        {
                            "flow": 20.0,
                            "outdoor": -15.0,
                        },
                        {
                            "flow": 20.0,
                            "outdoor": -15.0,
                        },
                    ],
                },
                "value": "HC1",
            },
        ],
        "target_temperature": [
            {
                "value": 10.81,
                "attributes": {
                    "lower_limit": "10",
                    "upper_limit": "90",
                },
            },
        ],
    },
    "solar_circuit": {},
    "switch_valve": {
        "position": [
            {
                "attributes": {
                    "lower_limit": "0",
                    "upper_limit": "2",
                },
                "value": "open",
            },
        ],
    },
    "external_heat_source": {},
    "photovoltaic": {},
}

read_data_expected_extra_attributes_3: str = heating_curve_points_expected_data

read_data_option_payload_4: list[dict[str, Any]] = [
    {
        "name": "APPL.CtrlAppl.sParam.options.systemNumberOfHeatPumps",
        "attributes": {
            "dynLowerLimit": 1,
            "dynUpperLimit": 1,
            "formatId": "fmt2p0",
            "longText": "Qty heat pumps",
            "lowerLimit": "0",
            "upperLimit": "4",
        },
        "value": "2",
    },
    {
        "name": "APPL.CtrlAppl.sParam.options.systemNumberOfHeatingCircuits",
        "attributes": {
            "dynLowerLimit": 1,
            "dynUpperLimit": 1,
            "formatId": "fmt2p0",
            "longText": "Qty HC",
            "lowerLimit": "0",
            "upperLimit": "8",
        },
        "value": "1",
    },
    {
        "name": "APPL.CtrlAppl.sParam.options.systemNumberOfSolarCircuits",
        "attributes": {
            "dynLowerLimit": 1,
            "dynUpperLimit": 1,
            "formatId": "fmt2p0",
            "longText": "Qty solar",
            "upperLimit": "4",
            "lowerLimit": "0",
        },
        "value": "2",
    },
    {
        "name": "APPL.CtrlAppl.sParam.options.systemNumberOfBuffers",
        "attributes": {
            "formatId": "fmt2p0",
            "longText": "Qty buffers",
            "upperLimit": "0",
            "lowerLimit": "0",
            "dynLowerLimit": 1,
            "dynUpperLimit": 1,
        },
        "value": "1",
    },
    {
        "name": "APPL.CtrlAppl.sParam.options.systemNumberOfHotWaterTanks",
        "attributes": {
            "dynLowerLimit": 1,
            "dynUpperLimit": 1,
            "formatId": "fmt2p0",
            "longText": "Qty HW tank",
            "lowerLimit": "0",
            "upperLimit": "4",
        },
        "value": "1",
    },
    {
        "name": "APPL.CtrlAppl.sParam.options.systemNumberOfExtHeatSources",
        "attributes": {
            "formatId": "fmt2p0",
            "longText": "Qty ext. heat sources",
            "upperLimit": "1",
            "lowerLimit": "0",
        },
        "value": "0",
    },
    {
        "name": "APPL.CtrlAppl.sParam.options.systemNumberOfSwitchValves",
        "attributes": {
            "formatId": "fmt2p0",
            "longText": "Qty switch valves",
            "upperLimit": "0",
            "lowerLimit": "0",
            "dynLowerLimit": 1,
            "dynUpperLimit": 1,
        },
        "value": "1",
    },
]

read_data_payload_4: list[dict[str, Any]] = [
    {
        "name": "APPL.CtrlAppl.sParam.bufferTank[0].topTemp.values.actValue",
        "attributes": {
            "formatId": "fmtTemp",
            "longText": "Temp. top act.",
            "unitId": "Temp",
            "upperLimit": "90",
            "lowerLimit": "5",
        },
        "value": "45.0273",
    },
    {
        "name": "APPL.CtrlAppl.sParam.solarCircuit[0].collectorTemp.values.actValue",
        "attributes": {
            "formatId": "fmtTemp",
            "longText": "Source temp.",
            "unitId": "Temp",
            "upperLimit": "90",
            "lowerLimit": "20",
        },
        "value": "22.426912",
    },
    {
        "name": "APPL.CtrlAppl.sParam.solarCircuit[1].collectorTemp.values.actValue",
        "attributes": {
            "formatId": "fmtTemp",
            "longText": "Source temp.",
            "unitId": "Temp",
            "upperLimit": "90",
            "lowerLimit": "20",
        },
        "value": "22.426912",
    },
    {
        "name": "APPL.CtrlAppl.sParam.genericHeat[0].referenceTemp.values.actValue",
        "attributes": {
            "formatId": "fmtTemp",
            "longText": "Temp. act.1",
            "unitId": "Temp",
        },
        "value": "55.753",
    },
    {
        "name": "APPL.CtrlAppl.sParam.genericHeat[1].referenceTemp.values.actValue",
        "attributes": {
            "formatId": "fmtTemp",
            "longText": "Temp. act.2",
            "unitId": "Temp",
        },
        "value": "45.753",
    },
    {
        "name": "APPL.CtrlAppl.sParam.genericHeat[2].referenceTemp.values.actValue",
        "attributes": {
            "formatId": "fmtTemp",
            "longText": "Temp. act.1",
            "unitId": "Temp",
        },
        "value": "53.753",
    },
    {
        "name": "APPL.CtrlAppl.sParam.genericHeat[3].referenceTemp.values.actValue",
        "attributes": {
            "formatId": "fmtTemp",
            "longText": "Temp. act.2",
            "unitId": "Temp",
        },
        "value": "43.753",
    },
    {
        "name": "APPL.CtrlAppl.sParam.genericHeat[0].param.setTempMax.value",
        "attributes": {
            "formatId": "fmtTemp",
            "longText": "Temp. nom. 1",
            "unitId": "Temp",
            "upperLimit": "90",
            "lowerLimit": "0",
        },
        "value": "35",
    },
    {
        "name": "APPL.CtrlAppl.sParam.genericHeat[1].param.setTempMax.value",
        "attributes": {
            "formatId": "fmtTemp",
            "longText": "Temp. nom. 2",
            "unitId": "Temp",
            "upperLimit": "90",
            "lowerLimit": "0",
        },
        "value": "45",
    },
    {
        "name": "APPL.CtrlAppl.sParam.genericHeat[2].param.setTempMax.value",
        "attributes": {
            "formatId": "fmtTemp",
            "longText": "Temp. nom. 1",
            "unitId": "Temp",
            "upperLimit": "90",
            "lowerLimit": "0",
        },
        "value": "35",
    },
    {
        "name": "APPL.CtrlAppl.sParam.genericHeat[3].param.setTempMax.value",
        "attributes": {
            "formatId": "fmtTemp",
            "longText": "Temp. nom. 2",
            "unitId": "Temp",
            "upperLimit": "90",
            "lowerLimit": "0",
        },
        "value": "45",
    },
    {
        "name": "APPL.CtrlAppl.sParam.heatCircuit[0].values.heatRequest",
        "attributes": {
            "unitId": "Enum",
            "upperLimit": "6",
            "lowerLimit": "0",
        },
        "value": "false",
    },
    {
        "name": "APPL.CtrlAppl.sParam.heatCircuit[1].values.heatRequest",
        "attributes": {
            "unitId": "Enum",
            "upperLimit": "6",
            "lowerLimit": "0",
        },
        "value": "true",
    },
    {
        "name": "APPL.CtrlAppl.sParam.heatCircuit[2].values.heatRequest",
        "attributes": {
            "unitId": "Enum",
            "upperLimit": "6",
            "lowerLimit": "0",
        },
        "value": "false",
    },
    {
        "name": "APPL.CtrlAppl.sParam.heatCircuit[3].values.heatRequest",
        "attributes": {
            "unitId": "Enum",
            "upperLimit": "6",
            "lowerLimit": "0",
        },
        "value": "true",
    },
    {
        "name": "APPL.CtrlAppl.sParam.heatpump[0].values.heatpumpSubState",
        "attributes": {
            "formatId": "fmtHPSubState",
            "longText": "Substate",
            "unitId": "Enum",
            "upperLimit": "32767",
            "lowerLimit": "0",
        },
        "value": "21",
    },
    {
        "name": "APPL.CtrlAppl.sParam.heatpump[1].values.heatpumpSubState",
        "attributes": {
            "formatId": "fmtHPSubState",
            "longText": "Substate",
            "unitId": "Enum",
            "upperLimit": "32767",
            "lowerLimit": "0",
        },
        "value": "5",
    },
]

read_data_expected_data_4: str = (
    '[{"name": "APPL.CtrlAppl.sParam.bufferTank[0].topTemp.values.actValue", "attr": "1"}, '
    '{"name": "APPL.CtrlAppl.sParam.solarCircuit[0].collectorTemp.values.actValue", "attr": "1"}, '
    '{"name": "APPL.CtrlAppl.sParam.solarCircuit[1].collectorTemp.values.actValue", "attr": "1"}, '
    '{"name": "APPL.CtrlAppl.sParam.genericHeat[0].referenceTemp.values.actValue", "attr": "1"}, '
    '{"name": "APPL.CtrlAppl.sParam.genericHeat[1].referenceTemp.values.actValue", "attr": "1"}, '
    '{"name": "APPL.CtrlAppl.sParam.genericHeat[2].referenceTemp.values.actValue", "attr": "1"}, '
    '{"name": "APPL.CtrlAppl.sParam.genericHeat[3].referenceTemp.values.actValue", "attr": "1"}, '
    '{"name": "APPL.CtrlAppl.sParam.genericHeat[0].param.setTempMax.value", "attr": "1"}, '
    '{"name": "APPL.CtrlAppl.sParam.genericHeat[1].param.setTempMax.value", "attr": "1"}, '
    '{"name": "APPL.CtrlAppl.sParam.genericHeat[2].param.setTempMax.value", "attr": "1"}, '
    '{"name": "APPL.CtrlAppl.sParam.genericHeat[3].param.setTempMax.value", "attr": "1"}, '
    '{"name": "APPL.CtrlAppl.sParam.genericHeat[0].values.heatRequest", "attr": "1"}, '
    '{"name": "APPL.CtrlAppl.sParam.genericHeat[1].values.heatRequest", "attr": "1"}, '
    '{"name": "APPL.CtrlAppl.sParam.genericHeat[2].values.heatRequest", "attr": "1"}, '
    '{"name": "APPL.CtrlAppl.sParam.genericHeat[3].values.heatRequest", "attr": "1"}, '
    '{"name": "APPL.CtrlAppl.sParam.heatpump[0].values.heatpumpSubState", "attr": "1"}, '
    '{"name": "APPL.CtrlAppl.sParam.heatpump[1].values.heatpumpSubState", "attr": "1"}]'
)

read_data_expected_response_4: dict[str, Any] = {
    "system": {},
    "buffer_tank": {
        "current_top_temperature": [
            {
                "attributes": {
                    "lower_limit": "5",
                    "upper_limit": "90",
                },
                "value": 45.03,
            },
        ],
    },
    "hot_water_tank": {},
    "heat_pump": {
        "substate": [
            {
                "attributes": {
                    "lower_limit": "0",
                    "upper_limit": "32767",
                },
                "value": "pressure_equalization",
            },
            {
                "attributes": {
                    "lower_limit": "0",
                    "upper_limit": "32767",
                },
                "value": "pressure_equalization",
            },
        ],
    },
    "heat_circuit": {},
    "solar_circuit": {
        "current_temperature": [
            [
                {
                    "attributes": {},
                    "value": 55.75,
                },
                {
                    "attributes": {},
                    "value": 45.75,
                },
            ],
            [
                {
                    "attributes": {},
                    "value": 53.75,
                },
                {
                    "attributes": {},
                    "value": 43.75,
                },
            ],
        ],
        "heat_request": [
            [
                {
                    "attributes": {
                        "lower_limit": "0",
                        "upper_limit": "6",
                    },
                    "value": "off",
                },
                {
                    "attributes": {
                        "lower_limit": "0",
                        "upper_limit": "6",
                    },
                    "value": "on",
                },
            ],
            [
                {
                    "attributes": {
                        "lower_limit": "0",
                        "upper_limit": "6",
                    },
                    "value": "off",
                },
                {
                    "attributes": {
                        "lower_limit": "0",
                        "upper_limit": "6",
                    },
                    "value": "on",
                },
            ],
        ],
        "target_temperature": [
            [
                {
                    "attributes": {
                        "lower_limit": "0",
                        "upper_limit": "90",
                    },
                    "value": 35.0,
                },
                {
                    "attributes": {
                        "lower_limit": "0",
                        "upper_limit": "90",
                    },
                    "value": 45.0,
                },
            ],
            [
                {
                    "attributes": {
                        "lower_limit": "0",
                        "upper_limit": "90",
                    },
                    "value": 35.0,
                },
                {
                    "attributes": {
                        "lower_limit": "0",
                        "upper_limit": "90",
                    },
                    "value": 45.0,
                },
            ],
        ],
        "source_temperature": [
            {
                "attributes": {
                    "lower_limit": "20",
                    "upper_limit": "90",
                },
                "value": 22.43,
            },
            {
                "attributes": {
                    "lower_limit": "20",
                    "upper_limit": "90",
                },
                "value": 22.43,
            },
        ],
    },
    "switch_valve": {},
    "external_heat_source": {},
    "photovoltaic": {},
}



"""All API Constants."""

from dataclasses import dataclass
from enum import Enum
from itertools import chain
from typing import Any
from typing import Final
from typing import TypeAlias

API_DEFAULT_TIMEOUT: int = 10


class EndpointPath:
    """The endpoint paths."""

    READ_WRITE_VARS: Final[str] = "/var/readWriteVars"
    READ_VAR_CHILDREN: Final[str] = "/var/readVarChildren"
    DEVICE_CONTROL: Final[str] = "/deviceControl"
    SW_UPDATE: Final[str] = "/swupdate"
    DATE_TIME: Final[str] = "/dateTime"


class BaseEnum(Enum):
    @classmethod
    def _missing_(cls, value: object) -> "BaseEnum":
        for member in cls:
            member_value = member._value_

            if isinstance(member_value, tuple):
                if value in member_value:
                    return member
            elif value == member_value:  # pragma: no cover
                return member

        msg: str = f"{value!r} is not a valid {cls.__name__}"
        raise ValueError(msg)


class BoolEnum(BaseEnum):
    """Bool enum for ON / OFF."""

    OFF = 0
    ON = 1


SystemHasPhotovoltaics: type[BoolEnum] = BoolEnum
HeatPumpCompressorUseNightSpeed: type[BoolEnum] = BoolEnum
HeatPumpHasPassiveCooling: type[BoolEnum] = BoolEnum
HeatPumpHasCompressorFailure: type[BoolEnum] = BoolEnum
HeatPumpHasSourceFailure: type[BoolEnum] = BoolEnum
HeatPumpHasSourceActuatorFailure: type[BoolEnum] = BoolEnum
HeatPumpHasThreePhaseFailure: type[BoolEnum] = BoolEnum
HeatPumpHasSourcePressureFailure: type[BoolEnum] = BoolEnum
HeatPumpHasVFDFailure: type[BoolEnum] = BoolEnum
HeatCircuitHasRoomTemperature: type[BoolEnum] = BoolEnum
HeatCircuitHasRoomHumidity: type[BoolEnum] = BoolEnum
HeatCircuitUseHeatingCurve: type[BoolEnum] = BoolEnum
SolarCircuitOperatingMode: type[BoolEnum] = BoolEnum
SolarCircuitConsumer1PrioritySolar: type[BoolEnum] = BoolEnum
SolarCircuitHeatRequest: type[BoolEnum] = BoolEnum
BufferTankHeatRequest: type[BoolEnum] = BoolEnum
BufferTankCoolRequest: type[BoolEnum] = BoolEnum
HotWaterTankHeatRequest: type[BoolEnum] = BoolEnum
HotWaterTankHotWaterFlow: type[BoolEnum] = BoolEnum
HotWaterTankCirculationPumpState: type[BoolEnum] = BoolEnum
HeatPumpHeatRequest: type[BoolEnum] = BoolEnum
ExternalHeatSourceOperatingMode: type[BoolEnum] = BoolEnum
ExternalHeatSourceHeatRequest: type[BoolEnum] = BoolEnum


class SystemOperatingMode(BaseEnum):
    """Available system operating modes."""

    SETUP = -1
    STANDBY = 0
    SUMMER = 1
    AUTO_HEAT = 2
    AUTO_COOL = 3
    AUTO = 4


class BufferTankOperatingMode(BaseEnum):
    """Available buffer tank operating modes."""

    OFF = 0
    ON = 1
    HEAT_UP = 2


class HotWaterTankOperatingMode(BaseEnum):
    """Available hot water tank operating modes."""

    OFF = 0
    AUTO = 1
    ON = 2
    HEAT_UP = 3


class HeatPumpState(BaseEnum):
    """Available heat pump stats."""

    STANDBY = 0
    FLOW = 1
    AUTO_HEAT = 2
    DEFROST = 3
    AUTO_COOL = 4
    INFLOW = 5
    PUMP_DOWN = 6
    SHUTDOWN = 7
    ERROR = 8


class HeatPumpSubState(BaseEnum):
    """Available heat pump sub stats."""

    NONE = 0
    OIL_PREHEATING = 1
    PUMP_PRE_RUN = (2, 3)
    RANDOM_DELAY = 4
    PRESSURE_EQUALIZATION = (5, 21, 22)
    DEFROST_PRE_FLOW = 6
    DEFROST_MONITORING = 7
    SNOW_DETECTION = 8
    FLUSHING = 9
    DEFROST_INITIALIZATION = 10
    PREHEAT_FLOW = 11
    DEFROST = 12
    DRIP = (13, 25)
    DEFROST_END = 14
    OPEN = (15, 16)
    COMPRESSOR_POST_RUN = 17
    PUMP_POST_RUN = 18
    LUBRICATION_PULSE = 19
    REDUCED_SPEED = (20, 26)
    COMPRESSOR_DELAY = 23
    DEFROST_VENTING = 24
    SWITCH_HEATING_COOLING = 27
    WAIT_FOR_COMPRESSOR = (28, 33)
    COMPRESSOR_STOP = 29
    BIVALENT_LOCK = 30
    LOCKED = 31
    RETURN_FLOW_OFF = 32
    MIXER_OPEN = 34
    ZONE_VALVE = 35
    ELECTRIC_DEFROST = 36
    COUNTERFLOW_VALVE = 37


class HeatPumpOperatingMode(BaseEnum):
    """Available heat pump operating modes."""

    OFF = 0
    ON = 1
    BACKUP = 2


class HeatCircuitOperatingMode(BaseEnum):
    """Available heat circuit operating modes."""

    OFF = 0
    AUTO = 1
    DAY = 2
    NIGHT = 3
    HOLIDAY = 4
    PARTY = 5
    EXTERNAL = 8
    ROOM_CONTROL = 9


class SolarCircuitPriority(BaseEnum):
    """Available solar circuit priority."""

    LOW = 14
    HIGH = 15


class HeatCircuitHeatRequest(BaseEnum):
    """Available heat circuit heat request stats."""

    OFF = 0
    ON = 1
    FLOW_OFF = 2
    TEMPORARY_OFF = 3
    ROOM_OFF = 4
    OUTDOOR_OFF = 5
    INFLOW_OFF = 6


class HeatCircuitCoolRequest(BaseEnum):
    """Available heat circuit cool request stats."""

    OFF = 0
    ON = 1
    FLOW_OFF = 2
    TEMPORARY_OFF = 3
    ROOM_OFF = 4
    OUTDOOR_OFF = 5
    INFLOW_OFF = 6


MIN_HEATING_CURVE_POINTS: Final[int] = 7
MAX_HEATING_CURVE_POINTS: Final[int] = 16


class SwitchValvePosition(BaseEnum):
    """Available switch valve positons."""

    NEUTRAL = 0
    OPEN = 1
    CLOSED = 2


PAYLOAD_PREFIX: Final[str] = "APPL.CtrlAppl"


@dataclass
class EndpointProperties:
    """Properties from an endpoint."""

    value: str
    value_type: type[float | int | str]
    human_readable: type[Enum] | None = None
    quantity: int = 1
    read_only: bool = True

    def __post_init__(self) -> None:
        if ".param." in self.value:
            self.read_only = False


@dataclass
class SolarCircuitPriority1Before2EndpointProperties(EndpointProperties):
    @staticmethod
    def helper(modes: list[int | None]) -> dict["Section", Any]:
        """Add extra calls."""
        # The switch in the Web HMI calls to API endpoints
        # If priority_1 is on then priority is 14 and if off the priority is 15

        priorities: list[int | None] = list(
            chain.from_iterable(
                (
                    (SolarCircuitPriority.LOW.value if m == 1 else SolarCircuitPriority.HIGH.value if m == 0 else None),
                    None,
                )
                for m in modes
            ),
        )

        return {SolarCircuit._PRIORITY: priorities}  # noqa: SLF001


class System(Enum):
    """The system endpoint settings."""

    BUFFER_TANK_NUMBERS = EndpointProperties(
        f"{PAYLOAD_PREFIX}.sParam.options.systemNumberOfBuffers",
        value_type=int,
    )
    HOT_WATER_TANK_NUMBERS = EndpointProperties(
        f"{PAYLOAD_PREFIX}.sParam.options.systemNumberOfHotWaterTanks",
        value_type=int,
    )
    HEAT_PUMP_NUMBERS = EndpointProperties(
        f"{PAYLOAD_PREFIX}.sParam.options.systemNumberOfHeatPumps",
        value_type=int,
    )
    HEAT_CIRCUIT_NUMBERS = EndpointProperties(
        f"{PAYLOAD_PREFIX}.sParam.options.systemNumberOfHeatingCircuits",
        value_type=int,
    )
    SOLAR_CIRCUIT_NUMBERS = EndpointProperties(
        f"{PAYLOAD_PREFIX}.sParam.options.systemNumberOfSolarCircuits",
        value_type=int,
    )
    EXTERNAL_HEAT_SOURCE_NUMBERS = EndpointProperties(
        f"{PAYLOAD_PREFIX}.sParam.options.systemNumberOfExtHeatSources",
        value_type=int,
    )
    SWITCH_VALVE_NUMBERS = EndpointProperties(
        f"{PAYLOAD_PREFIX}.sParam.options.systemNumberOfSwitchValves",
        value_type=int,
    )
    HAS_PHOTOVOLTAICS = EndpointProperties(
        f"{PAYLOAD_PREFIX}.sParam.options.hasPhotovoltaics",
        value_type=str,
        human_readable=SystemHasPhotovoltaics,
    )
    OPERATING_MODE = EndpointProperties(
        f"{PAYLOAD_PREFIX}.sParam.param.operatingMode",
        value_type=int,
        human_readable=SystemOperatingMode,
    )
    OUTDOOR_TEMPERATURE = EndpointProperties(
        f"{PAYLOAD_PREFIX}.sParam.outdoorTemp.values.actValue",
        value_type=float,
    )
    CPU_USAGE = EndpointProperties(
        f"{PAYLOAD_PREFIX}.sProcData.globalCpuTimePercent",
        value_type=float,
    )
    WEBVIEW_CPU_USAGE = EndpointProperties(
        f"{PAYLOAD_PREFIX}.sProcData.processStatus[0].cpuTimePercent",
        value_type=float,
    )
    WEBSERVER_CPU_USAGE = EndpointProperties(
        f"{PAYLOAD_PREFIX}.sProcData.processStatus[1].cpuTimePercent",
        value_type=float,
    )
    CONTROL_CPU_USAGE = EndpointProperties(
        f"{PAYLOAD_PREFIX}.sProcData.processStatus[2].cpuTimePercent",
        value_type=float,
    )
    RAM_USAGE = EndpointProperties(
        f"{PAYLOAD_PREFIX}.sProcData.RAMstatus.tmpfs",
        value_type=int,
    )
    FREE_RAM = EndpointProperties(
        f"{PAYLOAD_PREFIX}.sProcData.totFreeRAM",
        value_type=int,
    )


class BufferTank(Enum):
    """The buffer tank endpoint settings."""

    NAME = EndpointProperties(
        f"{PAYLOAD_PREFIX}.sParam.bufferTank[%s].param.name",
        value_type=str,
    )
    CURRENT_TOP_TEMPERATURE = EndpointProperties(
        f"{PAYLOAD_PREFIX}.sParam.bufferTank[%s].topTemp.values.actValue",
        value_type=float,
    )
    CURRENT_BOTTOM_TEMPERATURE = EndpointProperties(
        f"{PAYLOAD_PREFIX}.sParam.bufferTank[%s].midTemp.values.actValue",
        value_type=float,
    )
    OPERATING_MODE = EndpointProperties(
        f"{PAYLOAD_PREFIX}.sParam.bufferTank[%s].param.operatingMode",
        value_type=int,
        human_readable=BufferTankOperatingMode,
    )
    STANDBY_TEMPERATURE = EndpointProperties(
        f"{PAYLOAD_PREFIX}.sParam.bufferTank[%s].param.backupTemp",
        value_type=float,
    )
    TARGET_TEMPERATURE = EndpointProperties(
        f"{PAYLOAD_PREFIX}.sParam.bufferTank[%s].values.setTemp",
        value_type=float,
    )
    HEAT_REQUEST = EndpointProperties(
        f"{PAYLOAD_PREFIX}.sParam.bufferTank[%s].values.heatRequestTop",
        value_type=str,
        human_readable=BufferTankHeatRequest,
    )
    COOL_REQUEST = EndpointProperties(
        f"{PAYLOAD_PREFIX}.sParam.bufferTank[%s].values.coolRequestBot",
        value_type=str,
        human_readable=BufferTankCoolRequest,
    )


class HotWaterTank(Enum):
    """The hot water tank endpoint settings."""

    NAME = EndpointProperties(
        f"{PAYLOAD_PREFIX}.sParam.hotWaterTank[%s].param.name",
        value_type=str,
    )
    CURRENT_TEMPERATURE = EndpointProperties(
        f"{PAYLOAD_PREFIX}.sParam.hotWaterTank[%s].topTemp.values.actValue",
        value_type=float,
    )
    OPERATING_MODE = EndpointProperties(
        f"{PAYLOAD_PREFIX}.sParam.hotWaterTank[%s].param.operatingMode",
        value_type=int,
        human_readable=HotWaterTankOperatingMode,
    )
    STANDBY_TEMPERATURE = EndpointProperties(
        f"{PAYLOAD_PREFIX}.sParam.hotWaterTank[%s].param.reducedSetTempMax.value",
        value_type=float,
    )
    TARGET_TEMPERATURE = EndpointProperties(
        f"{PAYLOAD_PREFIX}.sParam.hotWaterTank[%s].param.normalSetTempMax.value",
        value_type=float,
    )
    HEAT_REQUEST = EndpointProperties(
        f"{PAYLOAD_PREFIX}.sParam.hotWaterTank[%s].values.heatRequestTop",
        value_type=str,
        human_readable=HotWaterTankHeatRequest,
    )
    HOT_WATER_FLOW = EndpointProperties(
        f"{PAYLOAD_PREFIX}.sParam.hotWaterTank[%s].FreshWater.freshWaterFlow.values.actValue",
        value_type=str,
        human_readable=HotWaterTankHotWaterFlow,
    )
    FRESH_WATER_MODULE_TEMPERATURE = EndpointProperties(
        f"{PAYLOAD_PREFIX}.sParam.hotWaterTank[%s].FreshWater.freshWaterTemp.values.actValue",
        value_type=float,
    )
    FRESH_WATER_MODULE_PUMP_SPEED = EndpointProperties(
        f"{PAYLOAD_PREFIX}.sParam.hotWaterTank[%s].FreshWater.freshWaterPump.values.setValueScaled",
        value_type=float,
    )
    CIRCULATION_RETURN_TEMPERATURE = EndpointProperties(
        f"{PAYLOAD_PREFIX}.sParam.hotWaterTank[%s].circTemp.values.actValue",
        value_type=float,
    )
    CIRCULATION_PUMP_STATE = EndpointProperties(
        f"{PAYLOAD_PREFIX}.sParam.hotWaterTank[%s].circPump.pump.values.setValueB",
        value_type=str,
        human_readable=HotWaterTankCirculationPumpState,
    )


class HeatPump(Enum):
    """The heat pump endpoint settings."""

    NAME = EndpointProperties(
        f"{PAYLOAD_PREFIX}.sParam.heatpump[%s].param.name",
        value_type=str,
    )
    STATE = EndpointProperties(
        f"{PAYLOAD_PREFIX}.sParam.heatpump[%s].values.heatpumpState",
        value_type=int,
        human_readable=HeatPumpState,
    )
    SUBSTATE = EndpointProperties(
        f"{PAYLOAD_PREFIX}.sParam.heatpump[%s].values.heatpumpSubState",
        value_type=int,
        human_readable=HeatPumpSubState,
    )
    OPERATING_MODE = EndpointProperties(
        f"{PAYLOAD_PREFIX}.sParam.heatpump[%s].param.operatingMode",
        value_type=int,
        human_readable=HeatPumpOperatingMode,
    )
    COMPRESSOR_USE_NIGHT_SPEED = EndpointProperties(
        f"{PAYLOAD_PREFIX}.sParam.heatpump[%s].HeatPumpPowerCtrl.param.useDayNightSpeed",
        value_type=str,
        human_readable=HeatPumpCompressorUseNightSpeed,
    )
    COMPRESSOR_NIGHT_SPEED = EndpointProperties(
        f"{PAYLOAD_PREFIX}.sParam.heatpump[%s].HeatPumpPowerCtrl.param.maxPowerScaledNight",
        value_type=float,
    )
    CIRCULATION_PUMP = EndpointProperties(
        f"{PAYLOAD_PREFIX}.sParam.heatpump[%s].CircPump.values.setValueScaled",
        value_type=float,
    )
    SOURCE_PUMP_SPEED = EndpointProperties(
        f"{PAYLOAD_PREFIX}.sParam.heatpump[%s].Source.values.setValueScaled",
        value_type=float,
    )
    FLOW_TEMPERATURE = EndpointProperties(
        f"{PAYLOAD_PREFIX}.sParam.heatpump[%s].TempHeatFlow.values.actValue",
        value_type=float,
    )
    RETURN_FLOW_TEMPERATURE = EndpointProperties(
        f"{PAYLOAD_PREFIX}.sParam.heatpump[%s].TempHeatReflux.values.actValue",
        value_type=float,
    )
    SOURCE_INPUT_TEMPERATURE = EndpointProperties(
        f"{PAYLOAD_PREFIX}.sParam.heatpump[%s].TempSourceIn.values.actValue",
        value_type=float,
    )
    SOURCE_OUTPUT_TEMPERATURE = EndpointProperties(
        f"{PAYLOAD_PREFIX}.sParam.heatpump[%s].TempSourceOut.values.actValue",
        value_type=float,
    )
    COMPRESSOR_INPUT_TEMPERATURE = EndpointProperties(
        f"{PAYLOAD_PREFIX}.sParam.heatpump[%s].TempCompressorIn.values.actValue",
        value_type=float,
    )
    COMPRESSOR_OUTPUT_TEMPERATURE = EndpointProperties(
        f"{PAYLOAD_PREFIX}.sParam.heatpump[%s].TempCompressorOut.values.actValue",
        value_type=float,
    )
    COMPRESSOR = EndpointProperties(
        f"{PAYLOAD_PREFIX}.sParam.heatpump[%s].Compressor.values.setValueScaled",
        value_type=float,
    )
    CONDENSER_TEMPERATURE = EndpointProperties(
        f"{PAYLOAD_PREFIX}.sParam.heatpump[%s].OverHeatCtrl.values.tempCond",
        value_type=float,
    )
    VAPORIZER_TEMPERATURE = EndpointProperties(
        f"{PAYLOAD_PREFIX}.sParam.heatpump[%s].OverHeatCtrl.values.tempVap",
        value_type=float,
    )
    TARGET_OVERHEATING = EndpointProperties(
        f"{PAYLOAD_PREFIX}.sParam.heatpump[%s].OverHeatCtrl.values.setOH",
        value_type=float,
    )
    CURRENT_OVERHEATING = EndpointProperties(
        f"{PAYLOAD_PREFIX}.sParam.heatpump[%s].OverHeatCtrl.values.actOH",
        value_type=float,
    )
    EXPANSION_VALVE_POSITION = EndpointProperties(
        f"{PAYLOAD_PREFIX}.sParam.heatpump[%s].OverHeatCtrl.values.stepperPos",
        value_type=int,
    )
    HIGH_PRESSURE = EndpointProperties(
        f"{PAYLOAD_PREFIX}.sParam.heatpump[%s].HighPressure.values.actValue",
        value_type=float,
    )
    LOW_PRESSURE = EndpointProperties(
        f"{PAYLOAD_PREFIX}.sParam.heatpump[%s].LowPressure.values.actValue",
        value_type=float,
    )
    HEAT_REQUEST = EndpointProperties(
        f"{PAYLOAD_PREFIX}.sParam.heatpump[%s].values.request",
        value_type=str,
        human_readable=HeatPumpHeatRequest,
    )
    # TOTAL_HEATING_ENERGY = EndpointProperties(
    #     f"{PAYLOAD_PREFIX}.sParam.heatpump[%s].HeatMeter.values.accumulatedHeat",
    #     value_type=float,
    # )
    # DAILY_HEATING_ENERGY = EndpointProperties(
    #     f"{PAYLOAD_PREFIX}.sParam.heatpump[%s].HeatMeter.values.heatDay",
    #     value_type=float,
    # )
    HEATING_POWER = EndpointProperties(
        f"{PAYLOAD_PREFIX}.sParam.heatpump[%s].HeatMeter.values.power",
        value_type=float,
    )
    HEATING_MASS_FLOW_RATE = EndpointProperties(
        f"{PAYLOAD_PREFIX}.sParam.heatpump[%s].HeatMeter.values.massFlow",
        value_type=float,
    )
    # TOTAL_COOLING_ENERGY = EndpointProperties(
    #     f"{PAYLOAD_PREFIX}.sParam.heatpump[%s].CoolMeter.values.accumulatedHeat",
    #     value_type=float,
    # )
    # DAILY_COOLING_ENERGY = EndpointProperties(
    #     f"{PAYLOAD_PREFIX}.sParam.heatpump[%s].CoolMeter.values.heatDay",
    #     value_type=float,
    # )
    # COOLING_POWER = EndpointProperties(
    #     f"{PAYLOAD_PREFIX}.sParam.heatpump[%s].CoolMeter.values.power",
    #     value_type=float,
    # )
    COOLING_MASS_FLOW_RATE = EndpointProperties(
        f"{PAYLOAD_PREFIX}.sParam.heatpump[%s].CoolMeter.values.massFlow",
        value_type=float,
    )
    # TOTAL_HOT_WATER_ENERGY = EndpointProperties(
    #     f"{PAYLOAD_PREFIX}.sParam.heatpump[%s].HotWaterMeter.values.accumulatedHeat",
    #     value_type=float,
    # )
    # DAILY_HOT_WATER_ENERGY = EndpointProperties(
    #     f"{PAYLOAD_PREFIX}.sParam.heatpump[%s].HotWaterMeter.values.heatDay",
    #     value_type=float,
    # )
    HOT_WATER_POWER = EndpointProperties(
        f"{PAYLOAD_PREFIX}.sParam.heatpump[%s].HotWaterMeter.values.power",
        value_type=float,
    )
    HOT_WATER_MASS_FLOW_RATE = EndpointProperties(
        f"{PAYLOAD_PREFIX}.sParam.heatpump[%s].HotWaterMeter.values.massFlow",
        value_type=float,
    )
    # TOTAL_COMPRESSOR_ENERGY = EndpointProperties(
    #     f"{PAYLOAD_PREFIX}.sParam.heatpump[%s].ElectricEnergyMeter.values.accumulatedHeat",
    #     value_type=float,
    # )
    # DAILY_COMPRESSOR_ENERGY = EndpointProperties(
    #     f"{PAYLOAD_PREFIX}.sParam.heatpump[%s].ElectricEnergyMeter.values.heatDay",
    #     value_type=float,
    # )
    COMPRESSOR_POWER = EndpointProperties(
        f"{PAYLOAD_PREFIX}.sParam.heatpump[%s].ElectricEnergyMeter.values.power",
        value_type=float,
    )
    COP = EndpointProperties(
        f"{PAYLOAD_PREFIX}.sParam.heatpump[%s].values.COP",
        value_type=float,
    )
    HEATING_ENERGY = EndpointProperties(
        f"{PAYLOAD_PREFIX}.sStatisticalData.heatpump[%s].consumption.heating.energy",
        value_type=float,
    )
    HEATING_ENERGY_CONSUMPTION = EndpointProperties(
        f"{PAYLOAD_PREFIX}.sStatisticalData.heatpump[%s].consumption.heating.electricalenergy",
        value_type=float,
    )
    HEATING_SPF = EndpointProperties(
        f"{PAYLOAD_PREFIX}.sStatisticalData.heatpump[%s].EnergyEfficiencyRatioHeat",
        value_type=float,
    )
    COOLING_ENERGY = EndpointProperties(
        f"{PAYLOAD_PREFIX}.sStatisticalData.heatpump[%s].consumption.cooling.energy",
        value_type=float,
    )
    COOLING_ENERGY_CONSUMPTION = EndpointProperties(
        f"{PAYLOAD_PREFIX}.sStatisticalData.heatpump[%s].consumption.cooling.electricalenergy",
        value_type=float,
    )
    COOLING_SPF = EndpointProperties(
        f"{PAYLOAD_PREFIX}.sStatisticalData.heatpump[%s].EnergyEfficiencyRatioCool",
        value_type=float,
    )
    HOT_WATER_ENERGY = EndpointProperties(
        f"{PAYLOAD_PREFIX}.sStatisticalData.heatpump[%s].consumption.domHotWater.energy",
        value_type=float,
    )
    HOT_WATER_ENERGY_CONSUMPTION = EndpointProperties(
        f"{PAYLOAD_PREFIX}.sStatisticalData.heatpump[%s].consumption.domHotWater.electricalenergy",
        value_type=float,
    )
    HOT_WATER_SPF = EndpointProperties(
        f"{PAYLOAD_PREFIX}.sStatisticalData.heatpump[%s].EnergyEfficiencyRatioDomHotWater",
        value_type=float,
    )
    TOTAL_THERMAL_ENERGY = EndpointProperties(
        f"{PAYLOAD_PREFIX}.sStatisticalData.heatpump[%s].consumption.energy",
        value_type=float,
    )
    TOTAL_ENERGY_CONSUMPTION = EndpointProperties(
        f"{PAYLOAD_PREFIX}.sStatisticalData.heatpump[%s].consumption.electricalenergy",
        value_type=float,
    )
    TOTAL_SPF = EndpointProperties(
        f"{PAYLOAD_PREFIX}.sStatisticalData.heatpump[%s].EnergyEfficiencyRatio",
        value_type=float,
    )
    HAS_PASSIVE_COOLING = EndpointProperties(
        f"{PAYLOAD_PREFIX}.sParam.options.heatpump[%s].hasPassiveCooling",
        value_type=str,
        human_readable=HeatPumpHasPassiveCooling,
    )
    OPERATING_TIME = EndpointProperties(
        f"{PAYLOAD_PREFIX}.sParam.heatpump[%s].operationalData.operationalTimeS",
        value_type=int,
    )
    MAX_RUNTIME = EndpointProperties(
        f"{PAYLOAD_PREFIX}.sParam.heatpump[%s].operationalData.maxRunTimeS",
        value_type=int,
    )
    ACTIVATION_COUNTER = EndpointProperties(
        f"{PAYLOAD_PREFIX}.sParam.heatpump[%s].operationalData.activationCounter",
        value_type=int,
    )
    HAS_COMPRESSOR_FAILURE = EndpointProperties(
        f"{PAYLOAD_PREFIX}.sParam.heatpump[%s].FailureCompressor.values.actValue",
        value_type=str,
        human_readable=HeatPumpHasCompressorFailure,
    )
    HAS_SOURCE_FAILURE = EndpointProperties(
        f"{PAYLOAD_PREFIX}.sParam.heatpump[%s].FailureSource.values.actValue",
        value_type=str,
        human_readable=HeatPumpHasSourceFailure,
    )
    HAS_SOURCE_ACTUATOR_FAILURE = EndpointProperties(
        f"{PAYLOAD_PREFIX}.sParam.heatpump[%s].FailureActuatorSource.values.actValue",
        value_type=str,
        human_readable=HeatPumpHasSourceActuatorFailure,
    )
    HAS_THREE_PHASE_FAILURE = EndpointProperties(
        f"{PAYLOAD_PREFIX}.sParam.heatpump[%s].FailureThreePhase.values.actValue",
        value_type=str,
        human_readable=HeatPumpHasThreePhaseFailure,
    )
    HAS_SOURCE_PRESSURE_FAILURE = EndpointProperties(
        f"{PAYLOAD_PREFIX}.sParam.heatpump[%s].FailureSrcPressure.values.actValue",
        value_type=str,
        human_readable=HeatPumpHasSourcePressureFailure,
    )
    HAS_VFD_FAILURE = EndpointProperties(
        f"{PAYLOAD_PREFIX}.sParam.heatpump[%s].FailureVFD.values.actValue",
        value_type=str,
        human_readable=HeatPumpHasVFDFailure,
    )


class HeatCircuit(Enum):
    """The heat circuit endpoint settings."""

    NAME = EndpointProperties(
        f"{PAYLOAD_PREFIX}.sParam.heatCircuit[%s].param.name",
        value_type=str,
    )
    HAS_ROOM_TEMPERATURE = EndpointProperties(
        f"{PAYLOAD_PREFIX}.sParam.options.heatCircuit[%s].hasRoomTemp",
        value_type=str,
        human_readable=HeatCircuitHasRoomTemperature,
    )
    ROOM_TEMPERATURE = EndpointProperties(
        f"{PAYLOAD_PREFIX}.sParam.heatCircuit[%s].tempRoom.values.actValue",
        value_type=float,
    )
    HAS_ROOM_HUMIDITY = EndpointProperties(
        f"{PAYLOAD_PREFIX}.sParam.options.heatCircuit[%s].hasRoomHumidity",
        value_type=str,
        human_readable=HeatCircuitHasRoomHumidity,
    )
    ROOM_HUMIDITY = EndpointProperties(
        f"{PAYLOAD_PREFIX}.sParam.heatCircuit[%s].humidityRoom.values.actValue",
        value_type=float,
    )
    DEW_POINT = EndpointProperties(
        f"{PAYLOAD_PREFIX}.sParam.heatCircuit[%s].dewPoint.values.actValue",
        value_type=float,
    )
    FLOW_TEMPERATURE_SETPOINT = EndpointProperties(
        f"{PAYLOAD_PREFIX}.sParam.heatCircuit[%s].values.flowSetTemp",
        value_type=float,
    )
    FLOW_TEMPERATURE = EndpointProperties(
        f"{PAYLOAD_PREFIX}.sParam.heatCircuit[%s].heatCircuitMixer.flowTemp.values.actValue",
        value_type=float,
    )
    MIXER_RETURN_FLOW_TEMPERATURE = EndpointProperties(
        f"{PAYLOAD_PREFIX}.sParam.heatCircuit[%s].heatCircuitMixer.refluxTemp.values.actValue",
        value_type=float,
    )
    RETURN_FLOW_TEMPERATURE = EndpointProperties(
        f"{PAYLOAD_PREFIX}.sParam.heatCircuit[%s].tempReflux.values.actValue",
        value_type=float,
    )
    TARGET_TEMPERATURE = EndpointProperties(
        f"{PAYLOAD_PREFIX}.sParam.heatCircuit[%s].values.setValue",
        value_type=float,
    )
    SELECTED_TARGET_TEMPERATURE = EndpointProperties(
        f"{PAYLOAD_PREFIX}.sParam.heatCircuit[%s].values.selectedSetTemp",
        value_type=float,
    )
    TARGET_TEMPERATURE_DAY = EndpointProperties(
        f"{PAYLOAD_PREFIX}.sParam.heatCircuit[%s].param.normalSetTemp",
        value_type=float,
    )
    HEATING_LIMIT_DAY = EndpointProperties(
        f"{PAYLOAD_PREFIX}.sParam.heatCircuit[%s].param.thresholdDayTemp.value",
        value_type=float,
    )
    TARGET_TEMPERATURE_NIGHT = EndpointProperties(
        f"{PAYLOAD_PREFIX}.sParam.heatCircuit[%s].param.reducedSetTemp",
        value_type=float,
    )
    HEATING_LIMIT_NIGHT = EndpointProperties(
        f"{PAYLOAD_PREFIX}.sParam.heatCircuit[%s].param.thresholdNightTemp.value",
        value_type=float,
    )
    TARGET_TEMPERATURE_AWAY = EndpointProperties(
        f"{PAYLOAD_PREFIX}.sParam.heatCircuit[%s].param.holidaySetTemp",
        value_type=float,
    )
    TARGET_TEMPERATURE_OFFSET = EndpointProperties(
        f"{PAYLOAD_PREFIX}.sParam.heatCircuit[%s].param.offsetRoomTemp",
        value_type=float,
    )
    OPERATING_MODE = EndpointProperties(
        f"{PAYLOAD_PREFIX}.sParam.heatCircuit[%s].param.operatingMode",
        value_type=int,
        human_readable=HeatCircuitOperatingMode,
    )
    HEAT_REQUEST = EndpointProperties(
        f"{PAYLOAD_PREFIX}.sParam.heatCircuit[%s].values.heatRequest",
        value_type=int,
        human_readable=HeatCircuitHeatRequest,
    )
    COOL_REQUEST = EndpointProperties(
        f"{PAYLOAD_PREFIX}.sParam.heatCircuit[%s].values.coolRequest",
        value_type=int,
        human_readable=HeatCircuitCoolRequest,
    )
    AWAY_START_DATE = EndpointProperties(
        f"{PAYLOAD_PREFIX}.sParam.heatCircuit[%s].param.holiday.start",
        value_type=int,
    )
    AWAY_END_DATE = EndpointProperties(
        f"{PAYLOAD_PREFIX}.sParam.heatCircuit[%s].param.holiday.stop",
        value_type=int,
    )
    HEATING_CURVE_OFFSET = EndpointProperties(
        f"{PAYLOAD_PREFIX}.sParam.heatCircuit[%s].param.heatCurveOffset",
        value_type=float,
    )
    HEATING_CURVE_SLOPE = EndpointProperties(
        f"{PAYLOAD_PREFIX}.sParam.heatCircuit[%s].param.heatCurveGradient",
        value_type=float,
    )
    USE_HEATING_CURVE = EndpointProperties(
        f"{PAYLOAD_PREFIX}.sParam.heatCircuit[%s].param.enableHeatCurveLinTab",
        value_type=str,
        human_readable=HeatCircuitUseHeatingCurve,
    )
    HEATING_CURVE = EndpointProperties(
        f"{PAYLOAD_PREFIX}.sParam.heatCircuit[%s].param.linTab.fileName",
        value_type=str,
    )
    PUMP_SPEED = EndpointProperties(
        f"{PAYLOAD_PREFIX}.sParam.heatCircuit[%s].pumpAO.values.setValueScaled",
        value_type=float,
    )


class LineTablePool(Enum):
    """Pool of heating curve points.

    This is not a real section and read and write are only allowed with `client.heat_circuit.get_heating_curves()`
    and `client.heat_circuit.set_heating_curves()`.

    """

    HEATING_CURVE_NAME = EndpointProperties(
        f"{PAYLOAD_PREFIX}.sParam.linTabPool[%s].name",
        value_type=str,
    )
    HEATING_CURVE_POINTS = EndpointProperties(
        f"{PAYLOAD_PREFIX}.sParam.linTabPool[%s].noOfPoints",
        value_type=int,
        read_only=False,
    )
    HEATING_CURVE_POINT_X = EndpointProperties(
        f"{PAYLOAD_PREFIX}.sParam.linTabPool[%s].points[%s].x",
        value_type=float,
        read_only=False,
    )
    HEATING_CURVE_POINT_Y = EndpointProperties(
        f"{PAYLOAD_PREFIX}.sParam.linTabPool[%s].points[%s].y",
        value_type=float,
        read_only=False,
    )
    SAVE_HEATING_CURVE = EndpointProperties(
        f"{PAYLOAD_PREFIX}.sParam.linTabPool[%s].verCnt",
        value_type=str,
        read_only=False,
    )


class SolarCircuit(Enum):
    """The solar circuit endpoint settings."""

    NAME = EndpointProperties(
        f"{PAYLOAD_PREFIX}.sParam.solarCircuit[%s].param.name",
        value_type=str,
    )
    OPERATING_MODE = EndpointProperties(
        f"{PAYLOAD_PREFIX}.sParam.solarCircuit[%s].param.operatingMode",
        value_type=int,
        human_readable=SolarCircuitOperatingMode,
    )
    SOURCE_TEMPERATURE = EndpointProperties(
        f"{PAYLOAD_PREFIX}.sParam.solarCircuit[%s].collectorTemp.values.actValue",
        value_type=float,
    )
    PUMP_1 = EndpointProperties(
        f"{PAYLOAD_PREFIX}.sParam.solarCircuit[%s].values.pump1",
        value_type=float,
    )
    PUMP_2 = EndpointProperties(
        f"{PAYLOAD_PREFIX}.sParam.solarCircuit[%s].values.pump2",
        value_type=float,
    )
    CURRENT_TEMPERATURE = EndpointProperties(
        f"{PAYLOAD_PREFIX}.sParam.genericHeat[%s].referenceTemp.values.actValue",
        value_type=float,
        quantity=2,
    )
    TARGET_TEMPERATURE = EndpointProperties(
        f"{PAYLOAD_PREFIX}.sParam.genericHeat[%s].param.setTempMax.value",
        value_type=float,
        quantity=2,
    )
    HEAT_REQUEST = EndpointProperties(
        f"{PAYLOAD_PREFIX}.sParam.genericHeat[%s].values.heatRequest",
        value_type=str,
        human_readable=SolarCircuitHeatRequest,
        quantity=2,
    )
    HEATING_ENERGY = EndpointProperties(
        f"{PAYLOAD_PREFIX}.sParam.solarCircuit[%s].heatMeter.values.accumulatedHeat",
        value_type=float,
    )
    DAILY_ENERGY = EndpointProperties(
        f"{PAYLOAD_PREFIX}.sParam.solarCircuit[%s].heatMeter.values.heatDay",
        value_type=float,
    )
    ACTUAL_POWER = EndpointProperties(
        f"{PAYLOAD_PREFIX}.sParam.solarCircuit[%s].heatMeter.values.power",
        value_type=float,
    )
    PRIORITY_1_BEFORE_2 = SolarCircuitPriority1Before2EndpointProperties(
        f"{PAYLOAD_PREFIX}.sParam.hmiRetainData.consumer1PrioritySolar[%s]",
        value_type=str,
        human_readable=SolarCircuitConsumer1PrioritySolar,
        read_only=False,
    )
    _PRIORITY = EndpointProperties(
        f"{PAYLOAD_PREFIX}.sParam.genericHeat[%s].param.priority",
        value_type=int,
        quantity=2,
    )


class ExternalHeatSource(Enum):
    OPERATING_MODE = EndpointProperties(
        f"{PAYLOAD_PREFIX}.sParam.extHeatSource[%s].param.operatingMode",
        value_type=int,
        human_readable=ExternalHeatSourceOperatingMode,
    )
    TARGET_TEMPERATURE = EndpointProperties(
        f"{PAYLOAD_PREFIX}.sParam.extHeatSource[%s].values.setTemp",
        value_type=float,
    )
    HEAT_REQUEST = EndpointProperties(
        f"{PAYLOAD_PREFIX}.sParam.extHeatSource[%s].DO.values.setValueB",
        value_type=str,
        human_readable=ExternalHeatSourceHeatRequest,
    )
    OPERATING_TIME = EndpointProperties(
        f"{PAYLOAD_PREFIX}.sParam.extHeatSource[%s].DO.operationalData.operationalTimeS",
        value_type=int,
    )
    MAX_RUNTIME = EndpointProperties(
        f"{PAYLOAD_PREFIX}.sParam.extHeatSource[%s].DO.operationalData.maxRunTimeS",
        value_type=int,
    )
    ACTIVATION_COUNTER = EndpointProperties(
        f"{PAYLOAD_PREFIX}.sParam.extHeatSource[%s].DO.operationalData.activationCounter",
        value_type=int,
    )


class SwitchValve(Enum):
    POSITION = EndpointProperties(
        f"{PAYLOAD_PREFIX}.sParam.switchvalve[%s].values.actPosition",
        value_type=int,
        human_readable=SwitchValvePosition,
    )


class Photovoltaic(Enum):
    EXCESS_POWER = EndpointProperties(
        f"{PAYLOAD_PREFIX}.sParam.photovoltaics.ElectricEnergyMeter.values.power",
        value_type=float,
    )
    DAILY_ENERGY = EndpointProperties(
        f"{PAYLOAD_PREFIX}.sParam.photovoltaics.ElectricEnergyMeter.values.heatDay",
        value_type=float,
    )
    TOTAL_ENERGY = EndpointProperties(
        f"{PAYLOAD_PREFIX}.sParam.photovoltaics.ElectricEnergyMeter.values.accumulatedHeat",
        value_type=float,
    )


class SectionPrefix(str, Enum):
    """Section prefixes."""

    SYSTEM = "system"
    BUFFER_TANK = "buffer_tank"
    HOT_WATER_TANK = "hot_water_tank"
    HEAT_PUMP = "heat_pump"
    HEAT_CIRCUIT = "heat_circuit"
    SOLAR_CIRCUIT = "solar_circuit"
    EXTERNAL_HEAT_SOURCE = "external_heat_source"
    SWITCH_VALVE = "switch_valve"
    PHOTOVOLTAIC = "photovoltaic"


Section: TypeAlias = (
    System
    | BufferTank
    | HotWaterTank
    | HeatPump
    | HeatCircuit
    | SolarCircuit
    | ExternalHeatSource
    | SwitchValve
    | Photovoltaic
)



import pytest
from aioresponses.core import aioresponses

from keba_keenergy_api.api import KebaKeEnergyAPI


@pytest.mark.happy
class TestHappyPathSwitchValveSection:
    @pytest.mark.asyncio
    @pytest.mark.parametrize(
        ("human_readable", "payload_value", "expected_value"),
        [(True, 1, "open"), (False, 0, 0), (True, 2, "closed")],
    )
    async def test_get_position(
        self,
        human_readable: bool,  # noqa: FBT001
        payload_value: int,
        expected_value: str,
    ) -> None:

        with aioresponses() as mock_keenergy_api:
            mock_keenergy_api.post(
                "http://mocked-host/var/readWriteVars",
                payload=[
                    {
                        "name": "APPL.CtrlAppl.sParam.switchvalve[0].values.actPosition",
                        "attributes": {
                            "formatId": "fmtSwitchValveStateV1",
                            "longText": "Valve pos.",
                            "unitId": "Enum",
                            "upperLimit": "2",
                            "lowerLimit": "0",
                        },
                        "value": f"{payload_value}",
                    },
                ],
                headers={"Content-Type": "application/json;charset=utf-8"},
            )

            client: KebaKeEnergyAPI = KebaKeEnergyAPI(host="mocked-host")
            data: int | str = await client.switch_valve.get_position(human_readable=human_readable)

            assert isinstance(data, (int | str))
            assert data == expected_value

            mock_keenergy_api.assert_called_once_with(
                url="http://mocked-host/var/readWriteVars",
                data='[{"name": "APPL.CtrlAppl.sParam.switchvalve[0].values.actPosition", "attr": "1"}]',
                method="POST",
                auth=None,
                ssl=False,
            )
